# Copyright (C) 2021-2022 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

import pytest

from otx.api.configuration.elements.metadata_keys import (
    AFFECTS_OUTCOME_OF,
    DEFAULT_VALUE,
    DESCRIPTION,
    EDITABLE,
    ENUM_NAME,
    HEADER,
    MAX_VALUE,
    MIN_VALUE,
    OPTIONS,
    TYPE,
    UI_RULES,
    VISIBLE_IN_UI,
    WARNING,
    allows_dictionary_values,
    allows_model_template_override,
)
from tests.unit.api.constants.components import OtxSdkComponent
from tests.unit.api.constants.requirements import Requirements


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestMetadataKeys:
    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_metadata_keys_constants(self):
        """
        <b>Description:</b>
        Check metadata_keys constants

        <b>Input data:</b>
        metadata_keys constants

        <b>Expected results:</b>
        Test passes if values of metadata_keys constants are equal to expected
        """
        assert DEFAULT_VALUE == "default_value"
        assert MIN_VALUE == "min_value"
        assert MAX_VALUE == "max_value"
        assert DESCRIPTION == "description"
        assert HEADER == "header"
        assert WARNING == "warning"
        assert EDITABLE == "editable"
        assert VISIBLE_IN_UI == "visible_in_ui"
        assert AFFECTS_OUTCOME_OF == "affects_outcome_of"
        assert UI_RULES == "ui_rules"
        assert TYPE == "type"
        assert OPTIONS == "options"
        assert ENUM_NAME == "enum_name"

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_allows_model_template_override(self):
        """
        <b>Description:</b>
        Check "allows_model_template_override" function

        <b>Input data:</b>
        "keyword" constant

        <b>Expected results:</b>
        Test passes if value returned by "allows_model_template_override" function is equal to expected

        <b>Steps</b>
        1. Check value returned by "allows_model_template_override" function for "keyword" that can be overridden
        2. Check value returned by "allows_model_template_override" function for "keyword" that can not be overridden
        """
        # Checking value returned by "allows_model_template_override" for "keyword" that can be overridden
        for keyword in [
            DEFAULT_VALUE,
            MIN_VALUE,
            MAX_VALUE,
            DESCRIPTION,
            HEADER,
            EDITABLE,
            WARNING,
            VISIBLE_IN_UI,
            OPTIONS,
            ENUM_NAME,
            UI_RULES,
            AFFECTS_OUTCOME_OF,
        ]:
            assert allows_model_template_override(keyword)
        # Checking value returned by "allows_model_template_override" for "keyword" that can not be overridden
        for keyword in [TYPE, "non-constant keyword"]:
            assert not allows_model_template_override(keyword)

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_allows_dictionary_values(self):
        """
        <b>Description:</b>
        Check "allows_dictionary_values" function

        <b>Input data:</b>
        "keyword" constant

        <b>Expected results:</b>
        Test passes if value returned by "allows_dictionary_values" function is equal to expected

        <b>Steps</b>
        1. Check value returned by "allows_dictionary_values" function for "keyword" that allowed to have a dictionary
        as its value
        2. Check value returned by "allows_dictionary_values" function for "keyword" that not allowed to have a
        dictionary as its value
        """
        # Checking value returned by "allows_dictionary_values" for "keyword" that allowed to have a dictionary as its
        # value
        for keyword in [UI_RULES, OPTIONS]:
            assert allows_dictionary_values(keyword)
        # Checking value returned by "allows_dictionary_values" for "keyword" that not allowed to have a dictionary as
        # its value
        for keyword in [
            DEFAULT_VALUE,
            MIN_VALUE,
            MAX_VALUE,
            DESCRIPTION,
            HEADER,
            WARNING,
            EDITABLE,
            VISIBLE_IN_UI,
            AFFECTS_OUTCOME_OF,
            TYPE,
            ENUM_NAME,
            "non-constant keyword",
        ]:
            assert not allows_dictionary_values(keyword)



"""This module implements activation map."""

# Copyright (C) 2021-2022 Intel Corporation
# SPDX-License-Identifier: Apache-2.0

from pathlib import Path
from typing import Union

import cv2
import numpy as np


def get_actmap(
    saliency_map: np.ndarray,
    output_res: Union[tuple, list],
) -> np.ndarray:
    """Get activation map (heatmap)  from saliency map.

    It will return activation map from saliency map

    Args:
        saliency_map (np.ndarray): Saliency map with pixel values from 0-255
        output_res (Union[tuple, list]): Output resolution

    Returns:
        saliency_map (np.ndarray): [H, W, 3] colormap, more red means more salient

    """
    if len(saliency_map.shape) == 3:
        saliency_map = saliency_map[0]

    saliency_map = cv2.resize(saliency_map, output_res)
    saliency_map = cv2.applyColorMap(saliency_map, cv2.COLORMAP_JET)
    return saliency_map


def get_input_names_list(input_path: Union[str, int], capture):
    """Lists the filenames of all inputs for demo."""

    # Web camera input
    if isinstance(input_path, int):
        return []
    if "DIR" in str(capture.get_type()):
        return [f.name for f in Path(input_path).iterdir() if f.is_file()]
    else:
        return [Path(input_path).name]


def dump_frames(saved_frames: list, output: str, input_path: Union[str, int], capture):
    """Saves images/videos with predictions from saved_frames to output folder with proper names."""

    if not saved_frames:
        return

    output_path = Path(output)
    if not output_path.exists():
        output_path.mkdir(parents=True)

    filenames = get_input_names_list(input_path, capture)

    if "VIDEO" in str(capture.get_type()):
        filename = filenames[0]
        w, h, _ = saved_frames[0].shape
        video_path = str(output_path / filename)
        codec = cv2.VideoWriter_fourcc(*"mp4v")
        out = cv2.VideoWriter(video_path, codec, capture.fps(), (h, w))
        for frame in saved_frames:
            out.write(frame)
        out.release()
        print(f"Video was saved to {video_path}")
    else:
        if len(filenames) < len(saved_frames):
            filenames = [f"output_{i}" for i, _ in enumerate(saved_frames)]
        for filename, frame in zip(filenames, saved_frames):
            image_path = str(output_path / filename)
            cv2.imwrite(image_path, frame)
            print(f"Image was saved to {image_path}")



# Copyright (C) 2023 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#
import pytest

from otx.cli.registry import Registry, find_and_parse_model_template
from tests.test_suite.e2e_test_system import e2e_pytest_unit


class TestRegistry:
    @pytest.fixture
    def mock_templates(self, mocker):
        template1 = mocker.MagicMock(framework="OTX001", task_type="Classification", model_template_id="001")
        template2 = mocker.MagicMock(framework="OTX002", task_type="Segmentation", model_template_id="002")
        template3 = mocker.MagicMock(framework="OTX001", task_type="Detection", model_template_id="003")
        return [template1, template2, template3]

    @e2e_pytest_unit
    def test_init(self, mocker, mock_templates):
        mock_glob = mocker.patch("glob.glob")
        mock_glob.return_value = ["./template.yaml"]
        mock_abspath = mocker.patch("os.path.abspath")
        mock_abspath.return_value = "./template.yaml"
        mock_parse_model_template = mocker.patch("otx.cli.registry.registry.parse_model_template")
        mock_parse_model_template.return_value = mock_templates[0]

        r = Registry(templates_dir=".")
        assert r.templates == [mock_templates[0]]

        r = Registry(templates_dir=".", experimental=True)
        assert r.templates == [mock_templates[0]] * 2

        r = Registry(templates=mock_templates)
        assert r.templates == mock_templates

    @e2e_pytest_unit
    def test_init_empty_raise_RuntimeError(self):
        with pytest.raises(RuntimeError):
            Registry()

    @e2e_pytest_unit
    def test_filter(self, mock_templates):
        """Check TestRegistry.filter function working well.

        <Steps>
            1. Create an instance of Registry with the mock templates
            2. Test filtering by framework
            3. Test filtering by task_type
            4. Test filtering by both framework and task_type
        """
        registry = Registry(templates=mock_templates)

        filtered_registry = registry.filter(framework="OTX001")
        assert len(filtered_registry.templates) == 2
        assert filtered_registry.templates[0].task_type == "Classification"
        assert filtered_registry.templates[1].task_type == "Detection"
        filtered_registry = registry.filter(framework="OTX002")
        assert len(filtered_registry.templates) == 1
        assert filtered_registry.templates[0].task_type == "Segmentation"

        filtered_registry = registry.filter(task_type="Detection")
        assert len(filtered_registry.templates) == 1
        assert filtered_registry.templates[0].task_type == "Detection"

        filtered_registry = registry.filter(framework="OTX002", task_type="Segmentation")
        assert len(filtered_registry.templates) == 1
        assert filtered_registry.templates[0].task_type == "Segmentation"

    @e2e_pytest_unit
    def test_get(self, mock_templates):
        registry = Registry(templates=mock_templates)
        template = registry.get("003")
        assert template.task_type == "Detection"

    @e2e_pytest_unit
    def test_get_unexpected(self, mock_templates):
        registry = Registry(templates=mock_templates)
        with pytest.raises(ValueError):
            registry.get("004")

    @e2e_pytest_unit
    def test_get_backbones(self, mocker, mock_templates):
        mock_get_backbone_list = mocker.patch("otx.cli.registry.registry.get_backbone_list")
        mock_get_backbone_list.return_value = ["resnet50", "resnet101"]
        r = Registry(templates=mock_templates)
        assert r.get_backbones(["torch", "tensorflow"]) == {
            "torch": ["resnet50", "resnet101"],
            "tensorflow": ["resnet50", "resnet101"],
        }


@e2e_pytest_unit
def test_find_and_parse_model_template(mocker):
    mock_template = mocker.MagicMock(framework="test", task_type="test", model_template_id="001")
    mock_parse_model_template = mocker.patch("otx.cli.registry.registry.parse_model_template")
    mock_parse_model_template.return_value = mock_template

    assert find_and_parse_model_template("001") == mock_template


@e2e_pytest_unit
def test_find_and_parse_model_template_exists_true(mocker):
    mock_template = mocker.MagicMock(framework="test", task_type="test", model_template_id="001")
    mock_parse_model_template = mocker.patch("otx.cli.registry.registry.parse_model_template")
    mock_parse_model_template.return_value = mock_template
    mock_exists = mocker.patch("os.path.exists")
    mock_exists.return_value = True
    assert find_and_parse_model_template("001") == mock_template



"""MobileNet-V3-Small for multi-label config."""

# pylint: disable=invalid-name

_base_ = ["../../../../recipes/stages/classification/multilabel/incremental.yaml", "../base/models/mobilenet_v3.py"]

model = dict(
    type="SAMImageClassifier",
    task="classification",
    head=dict(
        type="CustomMultiLabelNonLinearClsHead",
        loss=dict(
            type="AsymmetricLossWithIgnore",
            gamma_pos=0.0,
            gamma_neg=0.0,
        ),
    ),
)



# Copyright (C) 2021-2022 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#
import warnings

import cv2
import numpy as np
import pytest

from otx.api.entities.annotation import (
    Annotation,
    AnnotationSceneEntity,
    AnnotationSceneKind,
)
from otx.api.entities.dataset_item import DatasetItemEntity
from otx.api.entities.id import ID
from otx.api.entities.image import Image
from otx.api.entities.label import Domain, LabelEntity
from otx.api.entities.scored_label import ScoredLabel
from otx.api.entities.shapes.ellipse import Ellipse
from otx.api.entities.shapes.polygon import Point, Polygon
from otx.api.entities.shapes.rectangle import Rectangle
from otx.api.utils.segmentation_utils import (
    create_annotation_from_segmentation_map,
    create_hard_prediction_from_soft_prediction,
    get_subcontours,
    mask_from_annotation,
    mask_from_dataset_item,
)
from tests.unit.api.constants.components import OtxSdkComponent
from tests.unit.api.constants.requirements import Requirements


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestSegmentationUtils:
    @staticmethod
    def rectangle_label():
        return LabelEntity(
            name="Rectangle label",
            domain=Domain.SEGMENTATION,
            id=ID("1_rectangle_label"),
        )

    @staticmethod
    def ellipse_label():
        return LabelEntity(name="Ellipse label", domain=Domain.SEGMENTATION, id=ID("3_ellipse_label"))

    @staticmethod
    def polygon_label():
        return LabelEntity(name="Polygon label", domain=Domain.SEGMENTATION, id=ID("6_polygon_label"))

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_mask_from_annotation(self):
        """
        <b>Description:</b>
        Check "mask_from_annotation" function

        <b>Input data:</b>
        List with "Annotation" class objects, list with "LabelEntity" class objects, "width", "height"

        <b>Expected results:</b>
        Test passes if array returned by "mask_from_annotation" function is equal to expected
        """
        rectangle_label = self.rectangle_label()
        ellipse_label = self.ellipse_label()
        polygon_label = self.polygon_label()
        empty_rectangle_label = LabelEntity(
            name="Empty Rectangle label",
            domain=Domain.SEGMENTATION,
            is_empty=True,
            id=ID("2_empty_rectangle_label"),
        )
        empty_ellipse_label = LabelEntity(
            name="Empty Ellipse label",
            domain=Domain.SEGMENTATION,
            is_empty=True,
            id=ID("5_empty_ellipse_label"),
        )
        empty_polygon_label = LabelEntity(
            name="Empty Polygon label",
            domain=Domain.SEGMENTATION,
            is_empty=True,
            id=ID("7_empty_polygon_label"),
        )
        non_annotation_label = LabelEntity(
            name="Non-annotation label",
            domain=Domain.SEGMENTATION,
            id=ID("4_empty_annotation_label"),
        )
        rectangle_annotation = Annotation(
            shape=Rectangle(x1=0.5, y1=0.7, x2=0.9, y2=0.9),
            labels=[ScoredLabel(rectangle_label), ScoredLabel(empty_rectangle_label)],
        )
        ellipse_annotation = Annotation(
            shape=Ellipse(x1=0.5, y1=0.2, x2=0.9, y2=0.4),
            labels=[ScoredLabel(ellipse_label), ScoredLabel(empty_ellipse_label)],
        )
        polygon_shape = Polygon(
            points=[
                Point(x=0.1, y=0.1),
                Point(x=0.1, y=0.3),
                Point(x=0.3, y=0.4),
                Point(x=0.4, y=0.4),
                Point(x=0.4, y=0.1),
            ]
        )
        polygon_annotation = Annotation(
            shape=polygon_shape,
            labels=[ScoredLabel(polygon_label), ScoredLabel(empty_polygon_label)],
        )
        no_labels_annotation = Annotation(shape=Rectangle(x1=0.1, y1=0.8, x2=0.2, y2=0.9), labels=[])
        annotations = [
            rectangle_annotation,
            ellipse_annotation,
            polygon_annotation,
            no_labels_annotation,
        ]
        labels = [
            rectangle_label,
            empty_rectangle_label,
            ellipse_label,
            non_annotation_label,
            empty_ellipse_label,
            polygon_label,
            empty_polygon_label,
        ]
        expected_array = [
            [0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
            [0, 6, 6, 6, 6, 0, 0, 0, 0, 0],
            [0, 6, 6, 6, 6, 3, 3, 3, 3, 0],
            [0, 6, 6, 6, 6, 3, 3, 3, 3, 0],
            [0, 0, 0, 6, 6, 0, 0, 3, 0, 0],
            [0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0, 1, 1, 1, 1, 1],
            [0, 0, 0, 0, 0, 1, 1, 1, 1, 1],
            [0, 0, 0, 0, 0, 1, 1, 1, 1, 1],
        ]
        expected_mask = np.expand_dims(expected_array, axis=2)
        mask = mask_from_annotation(annotations=annotations, labels=labels, width=10, height=10)
        assert np.array_equal(mask, expected_mask)

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_mask_from_dataset_item(self):
        """
        <b>Description:</b>
        Check "mask_from_dataset_item" function

        <b>Input data:</b>
        "DatasetItemEntity" class object, list with "LabelEntity" class objects

        <b>Expected results:</b>
        Test passes if array returned by "mask_from_dataset_item" function is equal to expected
        """
        rectangle_label = self.rectangle_label()
        non_included_label = self.rectangle_label()
        ellipse_label = self.ellipse_label()
        polygon_label = self.polygon_label()
        rectangle_annotation = Annotation(
            shape=Rectangle(x1=0.5, y1=0.7, x2=0.9, y2=0.9),
            labels=[ScoredLabel(rectangle_label), ScoredLabel(non_included_label)],
        )
        ellipse_annotation = Annotation(
            shape=Ellipse(x1=0.5, y1=0.2, x2=0.9, y2=0.4),
            labels=[ScoredLabel(ellipse_label)],
        )
        polygon_shape = Polygon(
            points=[
                Point(x=0.1, y=0.1),
                Point(x=0.1, y=0.3),
                Point(x=0.3, y=0.4),
                Point(x=0.4, y=0.4),
                Point(x=0.4, y=0.1),
            ]
        )
        polygon_annotation = Annotation(shape=polygon_shape, labels=[ScoredLabel(polygon_label)])
        image = Image(np.random.randint(low=0, high=255, size=(480, 640, 3)))
        annotation_scene = AnnotationSceneEntity(
            annotations=[rectangle_annotation, ellipse_annotation, polygon_annotation],
            kind=AnnotationSceneKind.ANNOTATION,
        )
        dataset_item = DatasetItemEntity(media=image, annotation_scene=annotation_scene)
        labels = [rectangle_label, ellipse_label, polygon_label]
        expected_mask = mask_from_annotation(
            annotations=dataset_item.get_annotations(),
            labels=labels,
            width=640,
            height=480,
        )
        mask = mask_from_dataset_item(dataset_item=dataset_item, labels=labels)
        assert np.array_equal(mask, expected_mask)

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_create_hard_prediction_from_soft_prediction(self):
        """
        <b>Description:</b>
        Check "create_hard_prediction_from_soft_prediction" function

        <b>Input data:</b>
        "soft_prediction" nd.array, "soft_threshold" value, "blur_strength" value parameters

        <b>Expected results:</b>
        Test passes if array returned by "create_hard_prediction_from_soft_prediction" function is equal to expected

        <b>Steps</b>
        1. Check array returned by "create_hard_prediction_from_soft_prediction" function for 2-dimensional array
        specified as "soft_prediction" parameter
        2. Check array returned by "create_hard_prediction_from_soft_prediction" function for 3-dimensional array
        specified as "soft_prediction" parameter
        3. Check that "ValueError" exception is raised by "create_hard_prediction_from_soft_prediction" function when
        1-dimensional array specified as "soft_prediction" parameter
        """

        def generate_two_dimensional_hard_prediction(prediction, threshold, strength):
            prediction_copy = prediction.copy()
            soft_prediction_blurred = cv2.blur(prediction_copy, (strength, strength))
            hard_prediction = soft_prediction_blurred > threshold
            return hard_prediction

        def generate_three_dimensional_hard_prediction(prediction, threshold, strength):
            prediction_copy = prediction.copy()
            soft_prediction_blurred = cv2.blur(prediction_copy, (strength, strength))
            soft_prediction_blurred[soft_prediction_blurred < threshold] = 0
            hard_prediction = np.argmax(soft_prediction_blurred, axis=2)
            return hard_prediction

        # Checking array returned by "create_hard_prediction_from_soft_prediction" for 2-dimensional array
        # Default value of "blur_strength"
        soft_prediction = np.random.uniform(0, 1.0, size=(10, 15))
        soft_threshold = 0.5
        expected_hard_prediction = generate_two_dimensional_hard_prediction(
            prediction=soft_prediction, threshold=soft_threshold, strength=5
        )
        actual_hard_prediction = create_hard_prediction_from_soft_prediction(
            soft_prediction=soft_prediction, soft_threshold=soft_threshold
        )
        assert np.array_equal(actual_hard_prediction, expected_hard_prediction)
        # Specified value of "blur_strength"
        soft_prediction = np.random.uniform(0, 1.0, size=(10, 15))
        soft_threshold = 0.6
        blur_strength = 4
        expected_hard_prediction = generate_two_dimensional_hard_prediction(
            prediction=soft_prediction, threshold=soft_threshold, strength=blur_strength
        )
        actual_hard_prediction = create_hard_prediction_from_soft_prediction(
            soft_prediction=soft_prediction,
            soft_threshold=soft_threshold,
            blur_strength=blur_strength,
        )
        assert np.array_equal(actual_hard_prediction, expected_hard_prediction)
        # Checking array returned by "create_hard_prediction_from_soft_prediction" for 3-dimensional array
        # Default value of "blur_strength"
        soft_prediction = np.random.uniform(0, 1.0, size=(8, 10, 5))
        soft_threshold = 0.4
        expected_hard_prediction = generate_three_dimensional_hard_prediction(
            prediction=soft_prediction, threshold=soft_threshold, strength=5
        )
        actual_hard_prediction = create_hard_prediction_from_soft_prediction(
            soft_prediction=soft_prediction, soft_threshold=soft_threshold
        )
        assert np.array_equal(actual_hard_prediction, expected_hard_prediction)
        # Checking array returned by "create_hard_prediction_from_soft_prediction" for 3-dimensional array
        # Specified value of "blur_strength"
        soft_prediction = np.random.uniform(0, 1.0, size=(10, 5, 6))
        soft_threshold = 0.5
        blur_strength = 6
        expected_hard_prediction = generate_three_dimensional_hard_prediction(
            prediction=soft_prediction, threshold=soft_threshold, strength=blur_strength
        )
        actual_hard_prediction = create_hard_prediction_from_soft_prediction(
            soft_prediction=soft_prediction,
            soft_threshold=soft_threshold,
            blur_strength=blur_strength,
        )
        assert np.array_equal(actual_hard_prediction, expected_hard_prediction)
        # Checking that "ValueError" exception is raised by "create_hard_prediction_from_soft_prediction" when
        # 1-dimensional array specified as "soft_prediction"
        soft_prediction = np.random.uniform(0, 1.0, size=1)
        with pytest.raises(ValueError):
            create_hard_prediction_from_soft_prediction(soft_prediction, 0.5)

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_get_subcontours(self):
        """
        <b>Description:</b>
        Check "get_subcontours" function

        <b>Input data:</b>
        "Contour" list with coordinates

        <b>Expected results:</b>
        Test passes if list of "Contour" coordinates is equal to expected

        <b>Steps</b>
        1. Check list returned by "get_subcontours" function for closed Contour
        2. Check list returned by "get_subcontours" function for open Contour
        3. Check list returned by "get_subcontours" function for Contour with no intersections
        """
        # Checking list returned by "get_subcontours" for closed Contour
        contour = [
            (0.2, 0.1),  # first rectangle
            (0.2, 0.2),
            (0.2, 0.3),
            (0.3, 0.3),
            (0.3, 0.2),
            (0.3, 0.1),
            (0.2, 0.1),
            (0.3, 0.1),  # second rectangle
            (0.3, 0.2),
            (0.3, 0.3),
            (0.4, 0.3),
            (0.4, 0.2),
            (0.4, 0.1),
            (0.3, 0.1),
            (0.2, 0.1),
        ]
        assert get_subcontours(contour) == [
            [(0.3, 0.1), (0.3, 0.2), (0.3, 0.3), (0.4, 0.3), (0.4, 0.2), (0.4, 0.1)],
            [(0.2, 0.1), (0.2, 0.2), (0.2, 0.3)],
        ]

        # Checking "get_subcontours" for open Contour
        contour = [
            (0.4, 0.4),  # first rectangle
            (0.4, 0.5),
            (0.5, 0.5),
            (0.5, 0.4),
            (0.4, 0.4),
            (0.5, 0.4),  # second rectangle
            (0.5, 0.5),
            (0.6, 0.5),
            (0.6, 0.4),
        ]
        assert get_subcontours(contour) == [[(0.4, 0.4), (0.5, 0.4), (0.5, 0.5), (0.6, 0.5), (0.6, 0.4)]]
        # Checking "get_subcontours" for Contour with no intersections
        contour = [(0.1, 0.2), (0.1, 0.2), (0.1, 0.2), (0.1, 0.2)]
        assert get_subcontours(contour) == []

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_create_annotation_from_segmentation_map(self):
        """
        <b>Description:</b>
        Check "create_annotation_from_segmentation_map" function

        <b>Input data:</b>
        "hard_prediction" array, "soft_prediction" array, "label_map" dictionary

        <b>Expected results:</b>
        Test passes if "Annotations" list returned by "create_annotation_from_segmentation_map" function is
        equal to expected

        <b>Steps</b>
        1. Check "Annotations" list returned by "create_annotation_from_segmentation_map" function for 2-dimensional
        prediction arrays
        2. Check "Annotations" list returned by "create_annotation_from_segmentation_map" function for 3-dimensional
        prediction arrays
        3. Check "Annotations" list returned by "create_annotation_from_segmentation_map" function for prediction arrays
        with hole in segmentation mask
        """

        def check_annotation(
            annotation: Annotation,
            expected_points: list,
            expected_label: str,
            expected_probability: float,
        ):
            assert isinstance(annotation.shape, Polygon)
            assert annotation.shape.points == expected_points
            annotation_labels = annotation._Annotation__labels  # type: ignore[attr-defined]
            assert len(annotation_labels) == 1
            assert annotation_labels[0].label == expected_label
            assert round(annotation_labels[0].probability, 5) == expected_probability

        # Checking list returned by "create_annotation_from_segmentation_map" for 2-dimensional arrays
        soft_prediction = np.array(
            [
                (0.0, 0.1, 0.8, 0.3, 0.2),
                (0.1, 0.6, 0.7, 0.6, 0.2),
                (0.2, 0.9, 0.8, 0.8, 0.1),
                (0.2, 0.6, 0.9, 0.7, 0.1),
                (0.0, 0.1, 0.2, 0.0, 0.1),
            ]
        )
        hard_prediction = np.array(
            [
                (False, False, True, False, False),
                (False, True, True, True, False),
                (False, True, True, True, False),
                (False, True, True, True, False),
                (False, False, False, False, False),
            ]
        )
        labels = {
            False: "false_label",
            True: "true_label",
            2: "label_2",
        }
        annotations = create_annotation_from_segmentation_map(
            hard_prediction=hard_prediction,
            soft_prediction=soft_prediction,
            label_map=labels,
        )
        assert len(annotations) == 1  # 1 subcontour is created
        check_annotation(
            annotation=annotations[0],
            expected_points=[
                Point(0.4, 0.0),
                Point(0.2, 0.2),
                Point(0.2, 0.4),
                Point(0.2, 0.6),
                Point(0.4, 0.6),
                Point(0.6, 0.6),
                Point(0.6, 0.4),
                Point(0.6, 0.2),
            ],
            expected_label="true_label",
            expected_probability=0.7375,
        )
        # Checking list returned by "create_annotation_from_segmentation_map" for 3-dimensional arrays
        soft_prediction = np.array(
            [
                ([0.8, 0.1, 0.2], [0.9, 0.1, 0.2], [0.3, 0.2, 0.8], [0.1, 0.2, 0.8]),
                ([0.1, 0.8, 0.3], [0.0, 0.8, 0.2], [0.2, 0.1, 0.9], [0.0, 0.2, 0.8]),
                ([0.1, 0.7, 0.3], [0.3, 0.8, 0.2], [0.1, 0.2, 0.8], [0.4, 0.3, 0.7]),
                ([0.0, 1.0, 0.0], [0.1, 0.9, 0.1], [0.1, 0.1, 0.9], [0.2, 0.2, 0.8]),
            ]
        )
        hard_prediction = np.array([(0, 0, 2, 2), (1, 1, 2, 2), (1, 1, 2, 2), (1, 1, 2, 2)])
        labels = {0: "false_label", 1: "class_1", 2: "class_2"}
        annotations = create_annotation_from_segmentation_map(
            hard_prediction=hard_prediction,
            soft_prediction=soft_prediction,
            label_map=labels,
        )
        assert len(annotations) == 2  # 2 subcontours are created
        check_annotation(
            annotation=annotations[0],
            expected_points=[
                Point(0.0, 0.25),
                Point(0.0, 0.5),
                Point(0.0, 0.75),
                Point(0.25, 0.75),
                Point(0.25, 0.5),
                Point(0.25, 0.25),
            ],
            expected_label="class_1",
            expected_probability=0.83333,
        )
        check_annotation(
            annotation=annotations[1],
            expected_points=[
                Point(0.5, 0.0),
                Point(0.5, 0.25),
                Point(0.5, 0.5),
                Point(0.5, 0.75),
                Point(0.75, 0.75),
                Point(0.75, 0.5),
                Point(0.75, 0.25),
                Point(0.75, 0.0),
            ],
            expected_label="class_2",
            expected_probability=0.8125,
        )
        # Checking list returned by "create_annotation_from_segmentation_map" for prediction arrays with hole in
        # segmentation mask
        soft_prediction = np.array(
            [
                (0.9, 0.85, 0.9, 1.0, 0.85, 0.9, 0.95, 1.0),
                (0.95, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.85),
                (0.9, 0.0, 1.0, 0.9, 0.85, 0.9, 0.0, 0.9),
                (0.85, 0.0, 0.8, 0.0, 0.0, 0.95, 0.0, 0.9),
                (0.9, 0.0, 0.85, 0.0, 0.0, 1.0, 0.0, 0.9),
                (0.85, 0.0, 1.0, 0.9, 0.85, 0.9, 0.0, 0.85),
                (0.9, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.9),
                (0.9, 0.95, 1.0, 0.9, 0.95, 0.9, 0.95, 0.95),
            ]
        )
        hard_prediction = np.array(
            [
                (True, True, True, True, True, True, True, True),
                (True, False, False, False, False, False, False, True),
                (True, False, True, True, True, True, False, True),
                (True, False, True, False, False, True, False, True),
                (True, False, True, False, False, True, False, True),
                (True, False, True, True, True, True, False, True),
                (True, False, False, False, False, False, False, True),
                (True, True, True, True, True, True, True, True),
            ]
        )
        labels = {
            False: "false_label",
            True: "true_label",
            2: "label_2",
        }
        with warnings.catch_warnings():
            warnings.filterwarnings("ignore", "The geometry of the segmentation map")
            annotations = create_annotation_from_segmentation_map(
                hard_prediction=hard_prediction,
                soft_prediction=soft_prediction,
                label_map=labels,
            )
        assert len(annotations) == 2  # 2 subcontours are created
        check_annotation(
            annotation=annotations[0],
            expected_points=[
                Point(0.25, 0.25),
                Point(0.25, 0.375),
                Point(0.25, 0.5),
                Point(0.25, 0.625),
                Point(0.375, 0.625),
                Point(0.5, 0.625),
                Point(0.625, 0.625),
                Point(0.625, 0.5),
                Point(0.625, 0.375),
                Point(0.625, 0.25),
                Point(0.5, 0.25),
                Point(0.375, 0.25),
            ],
            expected_label="true_label",
            expected_probability=0.90833,
        )
        check_annotation(
            annotation=annotations[1],
            expected_points=[
                Point(0.0, 0.0),
                Point(0.0, 0.125),
                Point(0.0, 0.25),
                Point(0.0, 0.375),
                Point(0.0, 0.5),
                Point(0.0, 0.625),
                Point(0.0, 0.75),
                Point(0.0, 0.875),
                Point(0.125, 0.875),
                Point(0.25, 0.875),
                Point(0.375, 0.875),
                Point(0.5, 0.875),
                Point(0.625, 0.875),
                Point(0.75, 0.875),
                Point(0.875, 0.875),
                Point(0.875, 0.75),
                Point(0.875, 0.625),
                Point(0.875, 0.5),
                Point(0.875, 0.375),
                Point(0.875, 0.25),
                Point(0.875, 0.125),
                Point(0.875, 0.0),
                Point(0.75, 0.0),
                Point(0.625, 0.0),
                Point(0.5, 0.0),
                Point(0.375, 0.0),
                Point(0.25, 0.0),
                Point(0.125, 0.0),
            ],
            expected_label="true_label",
            expected_probability=0.91071,
        )



"""Unit test for otx.algorithms.common.adapters.mmcv.hooks.adaptive_training_hooks."""
# Copyright (C) 2023 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

from mmcv.runner import LrUpdaterHook
from mmcv.runner.hooks.checkpoint import CheckpointHook
from mmcv.runner.hooks.evaluation import EvalHook
from mmcv.utils import Config

from otx.algorithms.common.adapters.mmcv.hooks.adaptive_training_hook import (
    AdaptiveTrainSchedulingHook,
)
from otx.algorithms.common.adapters.mmcv.hooks.early_stopping_hook import (
    EarlyStoppingHook,
)
from tests.test_suite.e2e_test_system import e2e_pytest_unit


class MockEvalHook(EvalHook):
    def __init__(self):
        self.interval = 5
        self.start = 5
        self.by_epoch = False


class MockLrUpdaterHook(LrUpdaterHook):
    def __init__(self):
        self.interval = 1
        self.patience = 1


class MockEarlyStoppingHook(EarlyStoppingHook):
    def __init__(self):
        self.start = 1
        self.interval = 1
        self.patience = 1


class MockCheckpointHook(CheckpointHook):
    def __init__(self):
        self.interval = 1
        self.by_epoch = False


class TestAdaptiveTrainSchedulingHook:
    """Test class for AdaptiveTrainSchedulingHook."""

    @e2e_pytest_unit
    def test_before_run(self) -> None:
        """Test before_run function."""

        eval_hook = MockEvalHook()
        mock_runner = Config({"hooks": [None, eval_hook]})
        hook = AdaptiveTrainSchedulingHook(enable_eval_before_run=True)
        hook.before_run(mock_runner)
        assert eval_hook.interval == 1
        assert eval_hook.start == 0
        assert hook._original_interval == 5

    @e2e_pytest_unit
    def test_before_train_iter(self) -> None:
        """Test before_train_iter function."""

        hook = AdaptiveTrainSchedulingHook(enable_eval_before_run=True, enable_adaptive_interval_hook=True)
        hook._original_interval = 5

        eval_hook = MockEvalHook()
        lr_hook = MockLrUpdaterHook()
        early_hook = MockEarlyStoppingHook()
        ckpt_hook = MockCheckpointHook()

        mock_runner = Config(
            {
                "hooks": [eval_hook, lr_hook, early_hook, ckpt_hook],
                "max_epochs": 200,
                "max_iters": 200,
                "data_loader": range(10),
                "epoch": 5,
            }
        )

        hook.before_train_iter(mock_runner)
        assert hook._initialized is True
        assert hook.max_interval == 5
        assert hook._original_interval is None
        assert eval_hook.interval == 4
        assert lr_hook.interval == 4
        assert lr_hook.patience == 2
        assert early_hook.interval == 4
        assert early_hook.patience == 3
        assert ckpt_hook.interval == 4



# Copyright (C) 2021-2023 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

import numpy as np
import openvino.runtime as ov
import pytest
import torch

from otx.algorithms.detection.adapters.mmdet.models.dense_heads.mmov_ssd_head import (
    MMOVSSDHead,
)
from tests.test_suite.e2e_test_system import e2e_pytest_unit


class TestMMOVSSDHead:
    @pytest.fixture(autouse=True)
    def setup(self):

        params = []
        results = []

        param = ov.opset10.parameter([1, 24, 64, 64], ov.Type.f32, name="in")
        filter = ov.opset10.constant(np.random.normal(size=(1, 24, 64, 64)), ov.Type.f32)
        mul = ov.opset10.matmul(param, filter, False, False)
        result = ov.opset10.result(mul, name="reg_out")
        params.append(param)
        results.append(result)

        filter = ov.opset10.constant(np.random.normal(size=(1, 24, 64, 64)), ov.Type.f32)
        mul = ov.opset10.matmul(param, filter, False, False)
        result = ov.opset10.result(mul, name="cls_out")
        results.append(result)

        ov_model = ov.Model(results, params, "ssd_head")

        self.model = MMOVSSDHead(
            model_path_or_model=ov_model,
            transpose_reg=True,
            transpose_cls=True,
            background_index=0,
            inputs=dict(
                reg_convs=[
                    "in",
                ],
                cls_convs=[
                    "in",
                ],
            ),
            outputs=dict(
                reg_convs=[
                    "reg_out",
                ],
                cls_convs=[
                    "cls_out",
                ],
            ),
            num_classes=3,
            anchor_generator=dict(
                type="SSDAnchorGenerator",
                scale_major=False,
                input_size=300,
                basesize_ratio_range=(0.15, 0.9),
                strides=[24, 48, 92, 171, 400, 400],
                ratios=[[2], [2, 3], [2, 3], [2, 3], [2], [2]],
            ),
            bbox_coder=dict(
                type="DeltaXYWHBBoxCoder",
                target_means=[0.0, 0.0, 0.0, 0.0],
                target_stds=[0.1, 0.1, 0.2, 0.2],
            ),
        )

    @e2e_pytest_unit
    def test_init_weights(self):
        self.model.init_weights()

    @e2e_pytest_unit
    def test_forward(self):
        data = {}
        for key, shape in self.model.cls_convs[0].input_shapes.items():
            shape = [1 if i == -1 else i for i in shape]
            data[key] = torch.randn(shape)
        self.model.train()
        self.model(list(data.values()))



# Copyright (C) 2023 Intel Corporation
#
# SPDX-License-Identifier: MIT

import copy
import os
from functools import partial

import numpy as np
import pytest
import torch
from openvino.model_zoo.model_api.models import Model

from otx.algorithms.common.adapters.mmcv.utils.config_utils import MPAConfig
from otx.algorithms.detection.adapters.mmdet.task import MMDetectionTask
from otx.algorithms.detection.adapters.mmdet.utils import build_detector, patch_tiling
from otx.algorithms.detection.adapters.openvino.model_wrappers import OTXMaskRCNNModel
from otx.algorithms.detection.adapters.openvino.task import (
    OpenVINODetectionTask,
    OpenVINOMaskInferencer,
    OpenVINOTileClassifierWrapper,
)
from otx.algorithms.detection.configs.base import DetectionConfig
from otx.algorithms.detection.utils import generate_label_schema
from otx.api.configuration.helper import create
from otx.api.entities.label import LabelEntity
from otx.api.entities.model import ModelEntity
from otx.api.entities.model_template import (
    TaskType,
    parse_model_template,
    task_type_to_label_domain,
)
from otx.api.usecases.adapters.model_adapter import ModelAdapter
from tests.test_suite.e2e_test_system import e2e_pytest_unit
from tests.unit.algorithms.detection.test_helpers import (
    DEFAULT_ISEG_TEMPLATE_DIR,
    generate_det_dataset,
    init_environment,
)


class TestTilingTileClassifier:
    """Test the tile classifier"""

    @pytest.fixture(autouse=True)
    def setUp(self, otx_model) -> None:
        """Set up the test

        Args:
            otx_model (mocker): Mocked model
        """
        classes = ("rectangle", "ellipse", "triangle")
        self.ov_inferencer = dict()
        task_type = TaskType.INSTANCE_SEGMENTATION
        model_template = parse_model_template(os.path.join(DEFAULT_ISEG_TEMPLATE_DIR, "template.yaml"))
        self.hyper_parameters = create(model_template.hyper_parameters.data)
        self.hyper_parameters.tiling_parameters.enable_tiling = True
        self.hyper_parameters.tiling_parameters.enable_tile_classifier = True
        self.label_schema = generate_label_schema(classes, task_type_to_label_domain(task_type))
        self.task_env = init_environment(self.hyper_parameters, model_template, task_type=task_type)
        self.task_env.model = otx_model
        dataset, labels = generate_det_dataset(task_type=TaskType.INSTANCE_SEGMENTATION)
        self.dataset = dataset
        self.labels = labels

    @e2e_pytest_unit
    def test_openvino_sync(self, mocker):
        """Test OpenVINO tile classifier

        Args:
            mocker (_type_): pytest mocker from fixture
        """
        mocker.patch("otx.algorithms.detection.adapters.openvino.task.OpenvinoAdapter")
        mocker.patch.object(Model, "create_model", return_value=mocker.MagicMock(spec=OTXMaskRCNNModel))
        params = DetectionConfig(header=self.hyper_parameters.header)
        ov_mask_inferencer = OpenVINOMaskInferencer(params, self.label_schema, "")
        ov_mask_inferencer.model.resize_mask = False
        ov_mask_inferencer.model.preprocess.return_value = ({"foo": "bar"}, {"baz": "qux"})
        ov_mask_inferencer.model.postprocess.return_value = (
            np.array([], dtype=np.float32),
            np.array([], dtype=np.uint32),
            np.zeros((0, 4), dtype=np.float32),
            [],
        )
        ov_inferencer = OpenVINOTileClassifierWrapper(
            ov_mask_inferencer, tile_classifier_model_file="", tile_classifier_weight_file="", mode="sync"
        )
        ov_inferencer.model.__model__ = "OTX_MaskRCNN"
        mock_predict = mocker.patch.object(
            ov_inferencer.tiler.classifier, "infer_sync", return_value={"tile_prob": 0.5}
        )
        mocker.patch.object(OpenVINODetectionTask, "load_inferencer", return_value=ov_inferencer)
        ov_task = OpenVINODetectionTask(self.task_env)
        ov_task.inferencer.predict = partial(ov_task.inferencer.predict, mode="sync")
        updated_dataset = ov_task.infer(self.dataset)

        mock_predict.assert_called()
        for updated in updated_dataset:
            assert updated.annotation_scene.contains_any([LabelEntity(name=self.labels[0].name, domain="DETECTION")])

        output_model = copy.deepcopy(self.task_env.model)
        ov_task.model.set_data("openvino.bin", b"foo")
        ov_task.model.set_data("openvino.xml", b"bar")
        ov_task.model.set_data("tile_classifier.bin", b"foo")
        ov_task.model.set_data("tile_classifier.xml", b"bar")
        ov_task.deploy(output_model)
        assert output_model.exportable_code is not None

    @e2e_pytest_unit
    def test_load_tile_classifier_parameters(self, tmp_dir_path):
        """Test loading tile classifier parameters

        Args:
            tmp_dir_path (str): Path to temporary directory
        """
        maskrcnn_cfg = MPAConfig.fromfile(os.path.join(DEFAULT_ISEG_TEMPLATE_DIR, "model.py"))
        detector = build_detector(maskrcnn_cfg)
        model_ckpt = os.path.join(tmp_dir_path, "maskrcnn_without_tile_classifier.pth")
        torch.save({"state_dict": detector.state_dict()}, model_ckpt)

        # Enable tiling and save weights
        model_template = parse_model_template(os.path.join(DEFAULT_ISEG_TEMPLATE_DIR, "template.yaml"))
        hyper_parameters = create(model_template.hyper_parameters.data)
        hyper_parameters.tiling_parameters.enable_tiling = True
        hyper_parameters.tiling_parameters.enable_tile_classifier = True
        task_env = init_environment(hyper_parameters, model_template)
        output_model = ModelEntity(self.dataset, task_env.get_model_configuration())
        task = MMDetectionTask(task_env, output_path=str(tmp_dir_path))
        task._model_ckpt = model_ckpt
        task.save_model(output_model)
        for filename, model_adapter in output_model.model_adapters.items():
            with open(os.path.join(tmp_dir_path, filename), "wb") as write_file:
                write_file.write(model_adapter.data)

        # Read tiling parameters from weights
        with open(os.path.join(tmp_dir_path, "weights.pth"), "rb") as f:
            bin_data = f.read()
            model = ModelEntity(
                self.dataset,
                configuration=task_env.get_model_configuration(),
                model_adapters={"weights.pth": ModelAdapter(bin_data)},
            )
        task_env.model = model
        with pytest.raises(RuntimeError) as e:
            task = MMDetectionTask(task_env, output_path=str(tmp_dir_path))
            assert (
                str(e.value)
                == "Tile classifier is enabled but not found in the trained model. Please retrain your model."
            )

        maskrcnn_classifier_cfg = MPAConfig.fromfile(os.path.join(DEFAULT_ISEG_TEMPLATE_DIR, "model.py"))
        maskrcnn_classifier_cfg.model.type = "CustomMaskRCNNTileOptimized"
        tile_classifier_detector = build_detector(maskrcnn_classifier_cfg)
        tile_classifier_ckpt = os.path.join(tmp_dir_path, "maskrcnn_with_tile_classifier.pth")
        torch.save({"state_dict": tile_classifier_detector.state_dict()}, tile_classifier_ckpt)

        task_env = init_environment(hyper_parameters, model_template)
        output_model = ModelEntity(self.dataset, task_env.get_model_configuration())
        task = MMDetectionTask(task_env, output_path=str(tmp_dir_path))
        task._model_ckpt = tile_classifier_ckpt
        task.save_model(output_model)
        for filename, model_adapter in output_model.model_adapters.items():
            with open(os.path.join(tmp_dir_path, filename), "wb") as write_file:
                write_file.write(model_adapter.data)

        # Read tiling parameters from weights
        with open(os.path.join(tmp_dir_path, "weights.pth"), "rb") as f:
            bin_data = f.read()
            model = ModelEntity(
                self.dataset,
                configuration=task_env.get_model_configuration(),
                model_adapters={"weights.pth": ModelAdapter(bin_data)},
            )
        task_env.model = model
        task = MMDetectionTask(task_env, output_path=str(tmp_dir_path))

    @e2e_pytest_unit
    def test_patch_tiling_func(self):
        """Test that patch_tiling function works correctly"""
        cfg = MPAConfig.fromfile(os.path.join(DEFAULT_ISEG_TEMPLATE_DIR, "model.py"))
        model_template = parse_model_template(os.path.join(DEFAULT_ISEG_TEMPLATE_DIR, "template.yaml"))
        hyper_parameters = create(model_template.hyper_parameters.data)
        hyper_parameters.tiling_parameters.enable_tiling = True
        hyper_parameters.tiling_parameters.enable_tile_classifier = True
        patch_tiling(cfg, hyper_parameters, self.dataset)



# Copyright (C) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions
# and limitations under the License.

import pytest

from otx.api.entities.color import Color
from otx.api.entities.id import ID
from otx.api.entities.label import Domain, LabelEntity
from otx.api.entities.scored_label import LabelSource, ScoredLabel
from tests.unit.api.constants.components import OtxSdkComponent
from tests.unit.api.constants.requirements import Requirements


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestScoredLabel:
    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_scored_label(self):
        """
        <b>Description:</b>
        Check the ScoredLabel can correctly return the value

        <b>Input data:</b>
        LabelEntity

        <b>Expected results:</b>
        Test passes if the results match
        """
        car = LabelEntity(id=ID(123456789), name="car", domain=Domain.DETECTION, is_empty=False)
        person = LabelEntity(id=ID(987654321), name="person", domain=Domain.DETECTION, is_empty=False)
        car_label = ScoredLabel(car)
        person_label = ScoredLabel(person)

        for attr in ["id", "name", "color", "hotkey", "creation_date", "is_empty"]:
            assert getattr(car_label, attr) == getattr(car, attr)

        assert car_label.get_label() == car
        assert car_label == ScoredLabel(car)
        assert car_label != car
        assert car_label != person_label
        assert hash(car_label) == hash(str(car_label))

        probability = 0.0
        assert car_label.probability == probability
        delta_probability = 0.4
        probability += delta_probability
        car_label.probability += delta_probability
        assert car_label.probability == probability

        label_source = LabelSource()
        assert car_label.label_source == label_source
        user_name = "User Name"
        car_label.label_source.user_id = user_name
        label_source_with_user = LabelSource(user_id=user_name)
        assert car_label.label_source == label_source_with_user

        car.color = Color(red=16, green=15, blue=56, alpha=255)
        assert repr(car_label) == (
            "ScoredLabel(123456789, name=car, probability=0.4, domain=DETECTION, color="
            "Color(red=16, green=15, blue=56, alpha=255), hotkey=, "
            "label_source=LabelSource(user_id='User Name', model_id=ID(), "
            "model_storage_id=ID()))"
        )



# Copyright (C) 2021-2023 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

import os
import tempfile

from openvino.model_zoo._configuration import Model

from otx.core.ov.omz_wrapper import (
    download_model,
    get_model_configuration,
    get_omz_model,
)
from tests.test_suite.e2e_test_system import e2e_pytest_unit


@e2e_pytest_unit
def test_get_model_configuration():
    assert get_model_configuration("aa") is None
    model = get_model_configuration("mobilenet-v2-pytorch")
    assert isinstance(model, Model)
    assert getattr(model, "subdirectory") is not None
    assert getattr(model, "subdirectory_ori") is not None


@e2e_pytest_unit
def test_download_model():
    model = get_model_configuration("mobilenet-v2-pytorch")
    with tempfile.TemporaryDirectory() as tempdir:
        download_model(model, download_dir=tempdir)
        assert str(model.subdirectory) in os.listdir(tempdir)
        download_model(model, download_dir=tempdir)


@e2e_pytest_unit
def test_get_omz_model():
    with tempfile.TemporaryDirectory() as tempdir:
        model = get_omz_model("mobilenet-v2-pytorch", download_dir=tempdir, output_dir=tempdir)
        assert model is not None
        assert "model_path" in model and os.path.exists(model["model_path"])
        assert "weight_path" in model and os.path.exists(model["weight_path"])



# Copyright (C) 2021-2023 Intel Corporation
# SPDX-Lggggcense-Identifier: Apache-2.0
#

import pytest

from otx.core.ov.ops.object_detections import (
    DetectionOutputV0,
    PriorBoxClusteredV0,
    PriorBoxV0,
    ProposalV4,
    RegionYoloV0,
    ROIPoolingV0,
)
from tests.test_suite.e2e_test_system import e2e_pytest_unit


class TestProposalV4:
    @e2e_pytest_unit
    def test_invalid_attr(self):
        with pytest.raises(ValueError):
            ProposalV4(
                "dummy",
                shape=(1,),
                base_size=1,
                pre_nms_topn=1,
                post_nms_topn=1,
                nms_thresh=0.1,
                feat_stride=1,
                min_size=1,
                ratio=[1.0],
                scale=[1.0],
                framework="error",
            )

    @e2e_pytest_unit
    def test_forward(self):
        op = ProposalV4(
            "dummy",
            shape=(1,),
            base_size=1,
            pre_nms_topn=1,
            post_nms_topn=1,
            nms_thresh=0.1,
            feat_stride=1,
            min_size=1,
            ratio=[1.0],
            scale=[1.0],
        )
        with pytest.raises(NotImplementedError):
            op("dummy", "dummy", "dummy")


class TestROIPoolingV0:
    @e2e_pytest_unit
    def test_invalid_attr(self):
        with pytest.raises(ValueError):
            ROIPoolingV0(
                "dummy",
                shape=(1,),
                pooled_h=1,
                pooled_w=1,
                spatial_scale=1.0,
                method="error",
            )

    @e2e_pytest_unit
    def test_forward(self):
        op = ROIPoolingV0(
            "dummy",
            shape=(1,),
            pooled_h=1,
            pooled_w=1,
            spatial_scale=1.0,
        )
        with pytest.raises(NotImplementedError):
            op("dummy", "dummy")


class TestDetectionOutputV0:
    @e2e_pytest_unit
    def test_invalid_attr(self):
        with pytest.raises(ValueError):
            DetectionOutputV0(
                "dummy",
                shape=(1,),
                keep_top_k=[1],
                nms_threshold=0.1,
                code_type="error",
            )

    @e2e_pytest_unit
    def test_forward(self):
        op = DetectionOutputV0(
            "dummy",
            shape=(1,),
            keep_top_k=[1],
            nms_threshold=0.1,
        )
        with pytest.raises(NotImplementedError):
            op("dummy", "dummy", "dummy")


class TestRegionYoloV0:
    @e2e_pytest_unit
    def test_forward(self):
        op = RegionYoloV0(
            "dummy",
            shape=(1,),
            axis=1,
            coords=1,
            classes=1,
            end_axis=1,
            num=1,
        )
        with pytest.raises(NotImplementedError):
            op("dummy")


class TestPriorBoxV0:
    @e2e_pytest_unit
    def test_forward(self):
        op = PriorBoxV0("dummy", shape=(1,), offset=0.1)
        with pytest.raises(NotImplementedError):
            op("dummy", "dummy")


class TestPriorBoxClusteredV0:
    @e2e_pytest_unit
    def test_forward(self):
        op = PriorBoxClusteredV0("dummy", shape=(1,), offset=0.1)
        with pytest.raises(NotImplementedError):
            op("dummy", "dummy")



# Copyright (C) 2021-2023 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

import os
import tempfile

import numpy as np
import openvino.runtime as ov
import pytest
import torch

from otx.algorithms.detection.adapters.mmdet.models.dense_heads.mmov_yolov3_head import (
    MMOVYOLOV3Head,
)
from tests.test_suite.e2e_test_system import e2e_pytest_unit


class TestMMOVYOLOV3Head:
    @pytest.fixture(autouse=True)
    def setup(self):

        params = []
        results = []

        param = ov.opset10.parameter([1, 24, 64, 64], ov.Type.f32, name="in")
        params.append(param)
        constant = ov.opset10.constant(np.random.normal(size=(32, 24, 3, 3)), ov.Type.f32)
        node = ov.opset10.convolution(param, constant, [2, 2], [1, 1], [1, 1], [1, 1], "explicit", name="pred_in_1_tmp")
        result = ov.opset10.result(node, name="bridge_out_1")
        results.append(result)
        constant = ov.opset10.constant(np.random.normal(size=(32, 32, 3, 3)), ov.Type.f32)
        node = ov.opset10.convolution(node, constant, [2, 2], [1, 1], [1, 1], [1, 1], "explicit", name="pred_in_1")
        result = ov.opset10.result(node, name="pred_out_1")
        results.append(result)

        constant = ov.opset10.constant(np.random.normal(size=(16, 24, 3, 3)), ov.Type.f32)
        node = ov.opset10.convolution(param, constant, [2, 2], [1, 1], [1, 1], [1, 1], "explicit", name="pred_in_2_tmp")
        result = ov.opset10.result(node, name="bridge_out_2")
        results.append(result)
        constant = ov.opset10.constant(np.random.normal(size=(16, 16, 3, 3)), ov.Type.f32)
        node = ov.opset10.convolution(node, constant, [2, 2], [1, 1], [1, 1], [1, 1], "explicit", name="pred_in_2")
        result = ov.opset10.result(node, name="pred_out_2")
        results.append(result)

        ov_model = ov.Model(results, params, "yolov3_head")
        with tempfile.TemporaryDirectory() as tempdir:
            ov.serialize(ov_model, os.path.join(tempdir, "model.xml"), os.path.join(tempdir, "model.bin"))

            self.model = MMOVYOLOV3Head(
                model_path_or_model=os.path.join(tempdir, "model.xml"),
                inputs=dict(
                    convs_bridge=[
                        "in",
                        "in",
                    ],
                    convs_pred=[
                        "pred_in_1_tmp||pred_in_1",
                        "pred_in_2_tmp||pred_in_2",
                    ],
                ),
                outputs=dict(
                    convs_bridge=[
                        "bridge_out_1",
                        "bridge_out_2",
                    ],
                    convs_pred=[
                        "pred_out_1",
                        "pred_out_2",
                    ],
                ),
                anchor_generator=dict(
                    type="YOLOAnchorGenerator",
                    base_sizes=[[(81, 82), (135, 169), (344, 319)], [(23, 27), (37, 58), (81, 82)]],
                    strides=[32, 16],
                ),
                bbox_coder=dict(type="YOLOBBoxCoder"),
                featmap_strides=[32, 16],
                num_classes=80,
            )

    @e2e_pytest_unit
    def test_init_weights(self):
        self.model.init_weights()

    @e2e_pytest_unit
    def test_forward(self):
        data = []
        for conv_bridge in self.model.convs_bridge:
            for key, shape in conv_bridge.input_shapes.items():
                shape = [1 if i == -1 else i for i in shape]
                data.append(torch.randn(shape))
        self.model.train()
        self.model(data)



"""Fast RCNN for Action Detection."""

# Copyright (C) 2022 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions
# and limitations under the License.

import torch
from mmcv.runner import load_checkpoint
from mmcv.utils import ConfigDict
from mmdeploy.core import FUNCTION_REWRITER
from mmdet.models import DETECTORS
from mmdet.models.builder import build_detector
from mmdet.models.detectors import FastRCNN
from torch import nn

from otx.algorithms.action.configs.detection.base.faster_rcnn_config import (
    faster_rcnn,
    faster_rcnn_pretrained,
)


class ONNXPool3D(nn.Module):
    """3D pooling method for onnx export.

    ONNX dose not support dynamic pooling, therefore pooling operation should be changed into static way
    """

    def __init__(self, dim, pool_type):
        # TODO This is tempral solution to export two-stage action detection model.
        # This should be re-visited after fixing CVS-104657
        super().__init__()
        self.dim = dim
        if pool_type == "avg":
            self.pool = torch.mean
        else:
            self.pool = torch.max
        self.pool_type = pool_type

    def forward(self, x):
        """Forward method."""
        if self.dim == "temporal":
            if self.pool_type == "avg":
                return self.pool(x, 2, keepdim=True)
            return self.pool(x, 2, keepdim=True)[0]
        if self.pool_type == "avg":
            return self.pool(x, (3, 4), keepdim=True)
        return self.pool(self.pool(x, 3, keepdim=True)[0], 4, keepdim=True)[0]


@DETECTORS.register_module()
class AVAFastRCNN(FastRCNN):
    """Implementation of `Fast R-CNN` for Action Detection.

    Add forward_infer function for inference without pre-proposals
    """

    # pylint: disable=too-many-arguments, too-many-locals
    def __init__(self, backbone, roi_head, train_cfg, test_cfg, neck=None, pretrained=None):
        super(FastRCNN, self).__init__(
            backbone=backbone,
            neck=neck,
            roi_head=roi_head,
            train_cfg=train_cfg,
            test_cfg=test_cfg,
            pretrained=pretrained,
        )
        self.detector = None

    def patch_for_export(self):
        """Patch mmdetection's FastRCNN for exporting to onnx."""
        self._add_detector()
        self._patch_pools()

    def _add_detector(self):
        """Add Person Detector for inference.

        Action classification backbone + Fast RCNN structure use pre-proposals instead outputs from person detector
        This saves training and evaluation time. However, this method doesn't support inference for the dataset
        without pre-proposals. Therefore, when the model infers to dataset without pre-proposal, person detector
        should be added to action detector.
        """
        detector = ConfigDict(faster_rcnn)
        self.detector = build_detector(detector)
        ckpt = load_checkpoint(self.detector, faster_rcnn_pretrained, map_location="cpu")
        self.detector.CLASSES = ckpt["meta"]["CLASSES"]
        if self.detector.CLASSES[0] != "person":
            raise Exception(
                f"Person detector should have person as the first category, but got {self.detector.CLASSES}"
            )

    def _patch_pools(self):
        """Patch pooling functions for ONNX export.

        AVAFastRCNN's bbox head has pooling funcitons, which contain dynamic shaping.
        This funciton changes those pooling functions from dynamic shaping to static shaping.
        """
        self.roi_head.bbox_head.temporal_pool = ONNXPool3D("temporal", self.roi_head.bbox_head.temporal_pool_type)
        self.roi_head.bbox_head.spatial_pool = ONNXPool3D("spatial", self.roi_head.bbox_head.spatial_pool_type)

    # pylint: disable=no-self-argument
    @FUNCTION_REWRITER.register_rewriter(
        "otx.algorithms.action.adapters.mmaction.models.detectors.fast_rcnn.AVAFastRCNN.forward"
    )
    def forward_infer(ctx, self, imgs, img_metas):
        """Forward function for inference without pre-proposal."""
        clip_len = imgs.shape[2]
        img = imgs[:, :, int(clip_len / 2), :, :]
        det_bboxes, det_labels = self.detector.simple_test(img, img_metas[0])
        prediction = [det_bboxes[0][det_labels[0] == 0]]
        prediction = self.simple_test(imgs, img_metas[0], proposals=prediction)
        return prediction



"""Unit test for otx.algorithms.common.adapters.mmcv.hooks.cancel_interface_hook."""
# Copyright (C) 2023 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

from mmcv.runner import EpochBasedRunner

from otx.algorithms.common.adapters.mmcv.hooks.cancel_hook import CancelInterfaceHook
from tests.test_suite.e2e_test_system import e2e_pytest_unit


def mock_callback(*args):
    pass


class TestCancelInterfaceHook:
    """Test class for CancelInterfaceHook"""

    @e2e_pytest_unit
    def test_cancel(self):
        """Test cancel function."""

        class MockRunner(EpochBasedRunner):
            def __init__(self):
                self._max_epoch = 10
                self.should_stop = False
                self._epoch = 5

        hook = CancelInterfaceHook(mock_callback)
        assert hook.cancel() is None

        mock_runner = MockRunner()
        mock_runner.should_stop = True
        hook.runner = mock_runner
        assert hook.cancel() is None

        mock_runner.should_stop = False
        hook.runner = mock_runner
        hook.cancel()
        assert hook.runner.should_stop is True
        assert hook.runner._max_epochs == hook.runner.epoch

    @e2e_pytest_unit
    def test_before_run(self) -> None:
        hook = CancelInterfaceHook(mock_callback)
        runner = "RUNNER"

        hook.before_run(runner)
        assert hook.runner == "RUNNER"

    @e2e_pytest_unit
    def test_after_run(self) -> None:
        hook = CancelInterfaceHook(mock_callback)
        hook.runner = "RUNNER"
        hook.after_run("RUNNER")
        assert hook.runner is None



"""Initialzation OTX Tasks with MMCV framework."""
# Copyright (C) 2023 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

# flake8: noqa
import os

from .version import __version__, get_version


class MPAConstants:
    """Various path for MPA."""

    PACKAGE_ROOT = os.path.abspath(os.path.join(os.path.dirname(__file__), ".."))
    # PACKAGE_ROOT = os.path.dirname(Path(__file__).)
    RECIPES_PATH = os.path.join(PACKAGE_ROOT, "recipes")
    SAMPLES_PATH = os.path.join(PACKAGE_ROOT, "samples")
    MODELS_PATH = os.path.join(PACKAGE_ROOT, "models")


# print(f'pkg root ======> {MPAConstants.PACKAGE_ROOT}')

__all__ = [
    "get_version",
    "__version__",
    "MPAConstants",
]



# Copyright (C) 2023 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

from argparse import ArgumentParser, ArgumentTypeError
from pathlib import Path

import pytest

from otx.cli.utils import parser as target_package
from otx.cli.utils.parser import (
    MemSizeAction,
    add_hyper_parameters_sub_parser,
    gen_param_help,
    gen_params_dict_from_args,
    get_parser_and_hprams_data,
    str2bool,
)
from tests.test_suite.e2e_test_system import e2e_pytest_unit

FAKE_HYPER_PARAMETERS = {
    "a": {
        "a": "fake",
        "b": {
            "default_value": "default_value",
            "type": "SELECTABLE",
            "affects_outcome_of": "TRAINING",
            "header": "header",
        },
        "visible_in_ui": True,
    },
    "b": {
        "a": {
            "default_value": "default_value",
            "type": "BOOLEAN",
            "affects_outcome_of": "TRAINING",
            "header": "header",
        },
        "visible_in_ui": True,
    },
    "c": {
        "a": {
            "a": {
                "default_value": "default_value",
                "type": "INTEGER",
                "affects_outcome_of": "TRAINING",
                "header": "header",
            },
            "visible_in_ui": True,
        },
        "visible_in_ui": True,
    },
    "d": {
        "a": {"default_value": "default_value", "type": "FLOAT", "affects_outcome_of": "TRAINING", "header": "header"},
        "visible_in_ui": True,
    },
}


@e2e_pytest_unit
def test_gen_param_help():
    param_help = gen_param_help(FAKE_HYPER_PARAMETERS)

    hp_type_map = {
        "a.b": str,
        "b.a": bool,
        "c.a.a": int,
        "d.a": float,
    }
    for key, val in hp_type_map.items():
        assert param_help[key]["default"] == "default_value"
        assert param_help[key]["affects_outcome_of"] == "TRAINING"
        assert "help" in param_help[key]
        assert param_help[key]["type"] == val


@pytest.fixture
def mock_args(mocker):
    mock_args = mocker.Mock()
    setattr(mock_args, "params.a.a", 1)
    setattr(mock_args, "params.a.b", 2.1)
    setattr(mock_args, "params.a.c", True)
    setattr(mock_args, "params.b", "fake")
    setattr(mock_args, "params.c", 10)
    setattr(mock_args, "params.d", None)

    return mock_args


@e2e_pytest_unit
def test_gen_params_dict_from_args(mock_args):
    param_dict = gen_params_dict_from_args(
        mock_args, ["params.a.a", "params.a.b", "params.a.c", "params.b", "params.c"]
    )

    assert param_dict["a"]["a"]["value"] == 1
    assert param_dict["a"]["b"]["value"] == 2.1
    assert param_dict["a"]["c"]["value"] is True
    assert param_dict["b"]["value"] == "fake"
    assert param_dict["c"]["value"] == 10
    assert "d" not in param_dict


@e2e_pytest_unit
def test_gen_params_dict_from_args_with_type_hint(mock_args):
    type_hint = {
        "a.a": {"type": str},
        "a.b": {"type": int},
        "a.c": {"type": bool},
        "b": {"type": str},
        "c": {"type": str},
    }

    param_dict = gen_params_dict_from_args(
        mock_args, ["params.a.a", "params.a.b", "params.a.c", "params.b", "params.c"], type_hint
    )

    assert param_dict["a"]["a"]["value"] == "1"
    assert param_dict["a"]["b"]["value"] == 2
    assert param_dict["a"]["c"]["value"] is True
    assert param_dict["b"]["value"] == "fake"
    assert param_dict["c"]["value"] == "10"


@e2e_pytest_unit
@pytest.mark.parametrize("val", [True, False])
def test_str2bool_with_bool_input(val):
    assert str2bool(val) is val


@e2e_pytest_unit
@pytest.mark.parametrize("val", ["true", "1"])
def test_str2bool_with_bool_string_true(val):
    assert str2bool(val) is True


@e2e_pytest_unit
@pytest.mark.parametrize("val", ["false", "0"])
def test_str2bool_with_bool_string_false(val):
    assert str2bool(val) is False


@e2e_pytest_unit
@pytest.mark.parametrize("val", [1, 1.2, "abc"])
def test_str2bool_with_bool_wrong_input(val):
    with pytest.raises(ArgumentTypeError):
        assert str2bool(val)


@e2e_pytest_unit
def test_add_hyper_parameters_sub_parser(mocker):
    # prepare
    mock_parser = mocker.MagicMock()
    mock_subparser = mocker.MagicMock()
    mock_parser.add_subparsers.return_value.add_parser.return_value = mock_subparser

    # run
    parser = add_hyper_parameters_sub_parser(mock_parser, FAKE_HYPER_PARAMETERS, return_sub_parser=True)

    # check
    hp_type_map = {
        "a.b": str,
        "b.a": bool,
        "c.a.a": int,
        "d.a": float,
    }
    assert parser is not None
    add_args_call_args = mock_subparser.add_argument.call_args_list
    for i, key in enumerate(hp_type_map):
        assert add_args_call_args[i][0][0] == f"--{key}"
        assert add_args_call_args[i][1]["default"] == "default_value"
        assert "help" in add_args_call_args[i][1]
        assert add_args_call_args[i][1]["dest"] == f"params.{key}"

        hp_type = hp_type_map[key]
        if hp_type is not bool:
            assert add_args_call_args[i][1]["type"] == hp_type
        else:
            assert add_args_call_args[i][1]["type"] == str2bool


@e2e_pytest_unit
def test_get_parser_and_hprams_data_with_fake_template(mocker, tmp_dir):
    # prepare
    tmp_dir = Path(tmp_dir)
    fake_template_file = tmp_dir / "template.yaml"
    fake_template_file.write_text("fake")
    mock_argv = ["otx train", str(fake_template_file), "params", "--left-args"]
    mocker.patch("sys.argv", mock_argv)
    mock_template = mocker.patch.object(target_package, "find_and_parse_model_template")

    # run
    parser, hyper_parameters, params = get_parser_and_hprams_data()

    # check
    mock_template.assert_called_once()
    assert hyper_parameters == {}
    assert params == ["params", "--left-args"]
    assert isinstance(parser, ArgumentParser)


@e2e_pytest_unit
def test_get_parser_and_hprams_data(mocker):
    # prepare
    mock_argv = ["otx train", "params", "--left-args"]
    mocker.patch("sys.argv", mock_argv)

    # run
    parser, hyper_parameters, params = get_parser_and_hprams_data()

    # check
    assert hyper_parameters == {}
    assert params == ["params", "--left-args"]
    assert isinstance(parser, ArgumentParser)


@pytest.fixture
def fxt_argparse():
    parser = ArgumentParser()
    parser.add_argument(
        "--mem-cache-size",
        dest="params.algo_backend.mem_cache_size",
        action=MemSizeAction,
        type=str,
        required=False,
        default=0,
    )
    return parser


@pytest.mark.parametrize(
    "mem_size_arg,expected",
    [
        ("1561", 1561),
        ("121k", 121 * (2**10)),
        ("121kb", 121 * (10**3)),
        ("121kib", 121 * (2**10)),
        ("121m", 121 * (2**20)),
        ("121mb", 121 * (10**6)),
        ("121mib", 121 * (2**20)),
        ("121g", 121 * (2**30)),
        ("121gb", 121 * (10**9)),
        ("121gib", 121 * (2**30)),
        ("121as", None),
        ("121dddd", None),
    ],
)
def test_mem_size_parsing(fxt_argparse, mem_size_arg, expected):
    try:
        args = fxt_argparse.parse_args(["--mem-cache-size", mem_size_arg])
        assert getattr(args, "params.algo_backend.mem_cache_size") == expected
    except ValueError:
        assert expected is None



"""Normalization."""
# Copyright (C) 2023 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

import torch
import torch.nn.functional as F
from torch import nn


class OnnxLpNormalization(torch.autograd.Function):  # pylint: disable=abstract-method
    """OnnxLpNormalization."""

    @staticmethod
    def forward(ctx, x, axis=0, p=2, eps=1e-12):  # pylint: disable=unused-argument
        """Forward."""
        denom = x.norm(2, axis, True).clamp_min(eps).expand_as(x)
        return x / denom

    @staticmethod
    def symbolic(g, x, axis=0, p=2, eps=1e-12):  # pylint: disable=invalid-name, unused-argument
        """Symbolic onnxLpNormalization."""
        return g.op("LpNormalization", x, axis_i=int(axis), p_i=int(p))


def normalize(x, dim, p=2, eps=1e-12):
    """Normalize method."""
    if torch.onnx.is_in_onnx_export():
        return OnnxLpNormalization.apply(x, dim, p, eps)
    return F.normalize(x, dim=dim, p=p, eps=eps)


class Normalize(nn.Module):
    """Normalize."""

    def __init__(self, dim=1, p=2, eps=1e-12):
        super().__init__()

        self.dim = dim
        self.p = p
        self.eps = eps

    def forward(self, x):
        """Forward."""
        return normalize(x, self.dim, self.p, self.eps)



# Copyright (C) 2021-2023 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

import pytest
import torch

from otx.algorithms.classification.adapters.mmcls.models.heads.mmov_cls_head import (
    MMOVClsHead,
)
from tests.test_suite.e2e_test_system import e2e_pytest_unit
from tests.unit.core.ov.models.mmcls.test_helpers import create_ov_model


class TestMMOVClsHead:
    @pytest.fixture(autouse=True)
    def setup(self):
        ov_model = create_ov_model()
        self.model = MMOVClsHead(
            model_path_or_model=ov_model,
        )

    @e2e_pytest_unit
    def test_forward_train(self):
        data = {}
        for key, shape in self.model.model.input_shapes.items():
            shape = [1 if i == -1 else i for i in shape]
            data[key] = torch.randn(shape)

        output = self.model.forward_train(list(data.values()), torch.tensor([0]))
        assert "loss" in output

    @e2e_pytest_unit
    def test_simple_test(self):
        data = {}
        for key, shape in self.model.model.input_shapes.items():
            shape = [1 if i == -1 else i for i in shape]
            data[key] = torch.randn(shape)

        outputs = self.model.simple_test(list(data.values()))
        assert isinstance(outputs, list)
        assert len(outputs) == 1



# Copyright (C) 2020-2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions
# and limitations under the License.


import pytest

from otx.api.entities.coordinate import Coordinate
from tests.unit.api.constants.components import OtxSdkComponent
from tests.unit.api.constants.requirements import Requirements


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestCoordinate:
    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_coordinate(self):
        """
        <b>Description:</b>
        To test Coordinate class

        <b>Input data:</b>
        Three coordinate instances

        <b>Expected results:</b>
        1. It raises TypeError in case of attempt of instantiation with wrong number of parameters
        2. Fields of instances initialized with correct values
        3. repr method returns correct strings then used against each instance
        4. hash method works as expected
        5. as_tuple method works as expected
        6. as_int_tuple method works as expected
        7. '==' method works as expected

        <b>Steps</b>
        1. Attempt to create Coordinate with wrong parameters numbers
        2. Create three Coordinate:
            two of them with similar set of init values, third one with different one.
        3. Check repr method
        4. Check hash method
        5. Check as_tuple() method
        6. Check as_int_tuple() method
        7. Check __eq__ method
        """
        with pytest.raises(TypeError):
            Coordinate()

        with pytest.raises(TypeError):
            Coordinate(1)

        coord_a = Coordinate(x=0, y=0)
        coord_b = Coordinate(x=0.0, y=0.0)
        coord_c = Coordinate(x=1, y=1)

        assert isinstance(coord_a, Coordinate)
        assert coord_a.x == 0
        assert coord_a.y == 0
        assert repr(coord_a) == "Coordinate(x=0, y=0)"
        assert hash(coord_a) == coord_a.__hash__()
        assert coord_a.as_tuple() == (0, 0)
        assert coord_a.as_int_tuple() == (0, 0)

        assert isinstance(coord_b, Coordinate)
        assert coord_b.x == 0.0
        assert coord_b.y == 0.0
        assert repr(coord_b) == "Coordinate(x=0.0, y=0.0)"
        assert hash(coord_b) == coord_b.__hash__()
        assert coord_b.as_tuple() == (0.0, 0.0)
        assert coord_b.as_int_tuple() == (0, 0)

        assert isinstance(coord_c, Coordinate)
        assert coord_c.x == 1
        assert coord_c.y == 1
        assert repr(coord_c) == "Coordinate(x=1, y=1)"
        assert hash(coord_c) == coord_c.__hash__()
        assert coord_c.as_tuple() == (1, 1)
        assert coord_c.as_int_tuple() == (1, 1)

        assert coord_a == coord_b != coord_c



"""Collection of transfrom pipelines for segmentation task."""

# Copyright (C) 2023 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

from copy import deepcopy
from typing import Any, Dict, List

import mmcv
import numpy as np
from mmcv.parallel import DataContainer as DC
from mmcv.utils import build_from_cfg
from mmseg.datasets.builder import PIPELINES
from mmseg.datasets.pipelines import Compose
from mmseg.datasets.pipelines.formatting import to_tensor
from PIL import Image
from torchvision import transforms as T
from torchvision.transforms import functional as F


@PIPELINES.register_module(force=True)
class Normalize:
    """Normalize the image.

    Added key is "img_norm_cfg".

    Args:
        mean (sequence): Mean values of 3 channels.
        std (sequence): Std values of 3 channels.
        to_rgb (bool): Whether to convert the image from BGR to RGB,
            default is true.
    """

    def __init__(self, mean, std, to_rgb=True):
        self.mean = np.array(mean, dtype=np.float32)
        self.std = np.array(std, dtype=np.float32)
        self.to_rgb = to_rgb

    def __call__(self, results):
        """Call function to normalize images.

        Args:
            results (dict): Result dict from loading pipeline.

        Returns:
            dict: Normalized results, 'img_norm_cfg' key is added into
                result dict.
        """

        for target in ["img", "ul_w_img", "aux_img"]:
            if target in results:
                results[target] = mmcv.imnormalize(results[target], self.mean, self.std, self.to_rgb)
        results["img_norm_cfg"] = dict(mean=self.mean, std=self.std, to_rgb=self.to_rgb)

        return results

    def __repr__(self):
        """Repr."""
        repr_str = self.__class__.__name__
        repr_str += f"(mean={self.mean}, std={self.std}, to_rgb=" f"{self.to_rgb})"
        return repr_str


@PIPELINES.register_module(force=True)
class DefaultFormatBundle:
    """Default formatting bundle.

    It simplifies the pipeline of formatting common fields, including "img"
    and "gt_semantic_seg". These fields are formatted as follows.

    - img: (1)transpose, (2)to tensor, (3)to DataContainer (stack=True)
    - gt_semantic_seg: (1)unsqueeze dim-0 (2)to tensor,
                       (3)to DataContainer (stack=True)
    """

    def __call__(self, results):
        """Call function to transform and format common fields in results.

        Args:
            results (dict): Result dict contains the data to convert.

        Returns:
            dict: The result dict contains the data that is formatted with
                default bundle.
        """
        for target in ["img", "ul_w_img", "aux_img"]:
            if target not in results:
                continue

            img = results[target]
            if len(img.shape) < 3:
                img = np.expand_dims(img, -1)

            if len(img.shape) == 3:
                img = np.ascontiguousarray(img.transpose(2, 0, 1)).astype(np.float32)
            elif len(img.shape) == 4:
                # for selfsl or supcon
                img = np.ascontiguousarray(img.transpose(0, 3, 1, 2)).astype(np.float32)
            else:
                raise ValueError(f"img.shape={img.shape} is not supported.")

            results[target] = DC(to_tensor(img), stack=True)

        for trg_name in ["gt_semantic_seg", "gt_class_borders", "pixel_weights"]:
            if trg_name not in results:
                continue

            out_type = np.float32 if trg_name == "pixel_weights" else np.int64
            results[trg_name] = DC(to_tensor(results[trg_name][None, ...].astype(out_type)), stack=True)

        return results

    def __repr__(self):
        """Repr."""
        return self.__class__.__name__


@PIPELINES.register_module()
class BranchImage:
    """Branch images by copying with name of key.

    Args:
        key_map (dict): keys to name each image.
    """

    def __init__(self, key_map):
        self.key_map = key_map

    def __call__(self, results):
        """Call function to branch images in img_fields in results.

        Args:
            results (dict): Result dict contains the image data to branch.

        Returns:
            dict: The result dict contains the original image data and copied image data.
        """
        for key1, key2 in self.key_map.items():
            if key1 in results:
                results[key2] = results[key1]
            if key1 in results["img_fields"]:
                results["img_fields"].append(key2)
        return results

    def __repr__(self):
        """Repr."""

        repr_str = self.__class__.__name__
        return repr_str


@PIPELINES.register_module()
class TwoCropTransform:
    """TwoCropTransform to combine two pipelines.

    Through `TwoCropTransformHook`, how frequently both pipelines (view0 + view1) is applied can be set.

    Args:
        view0 (list): Pipeline for online network.
        view1 (list): Pipeline for target network.
    """

    def __init__(self, view0: List, view1: List):
        self.view0 = Compose([build_from_cfg(p, PIPELINES) for p in view0])
        self.view1 = Compose([build_from_cfg(p, PIPELINES) for p in view1])
        self.is_both = True

    def __call__(self, results: Dict[str, Any]):
        """Callback function of TwoCropTransform.

        Args:
            results (dict): Inputs to be transformed.

        Returns:
            dict: Dictionary that includes stuffs for training.
                  They have different shape or attribute depending on `self.is_both`.
        """
        if self.is_both:
            results1 = self.view0(deepcopy(results))
            results2 = self.view1(deepcopy(results))

            results = deepcopy(results1)
            results["img"] = np.stack((results1["img"], results2["img"]), axis=0)
            results["gt_semantic_seg"] = np.stack((results1["gt_semantic_seg"], results2["gt_semantic_seg"]), axis=0)
            results["flip"] = [results1["flip"], results2["flip"]]

        else:
            results = self.view0(results)

        results["is_both"] = self.is_both

        return results


@PIPELINES.register_module
class RandomResizedCrop(T.RandomResizedCrop):
    """Wrapper for RandomResizedCrop in torchvision.transforms.

    Since this transformation is applied to PIL Image,
    `NDArrayToPILImage` must be applied first before this is applied.
    """

    def __call__(self, results: Dict[str, Any]):
        """Callback function of RandomResizedCrop.

        Args:
            results (dict): Inputs to be transformed.

        Returns:
            dict: Dictionary that includes transformed img and related information.
        """
        img = results["img"]
        i, j, height, width = self.get_params(img, self.scale, self.ratio)
        img = F.resized_crop(img, i, j, height, width, self.size, self.interpolation)
        results["img"] = img
        results["img_shape"] = img.size
        for key in results.get("seg_fields", []):
            results[key] = np.array(
                F.resized_crop(Image.fromarray(results[key]), i, j, height, width, self.size, self.interpolation)
            )

        # change order because of difference between numpy and PIL
        w_scale = results["img_shape"][0] / results["ori_shape"][1]
        h_scale = results["img_shape"][1] / results["ori_shape"][0]
        results["scale_factor"] = np.array([w_scale, h_scale, w_scale, h_scale], dtype=np.float32)

        return results


@PIPELINES.register_module
class RandomColorJitter(T.ColorJitter):
    """Wrapper for ColorJitter in torchvision.transforms.

    Since this transformation is applied to PIL Image,
    `NDArrayToPILImage` must be applied first before this is applied.

    Args:
        p (float): Probability for transformation. Defaults to 0.8.
    """

    def __init__(self, p: float = 0.8, **kwargs):
        super().__init__(**kwargs)
        assert 0 <= p <= 1
        self.p = p

    def __call__(self, results: Dict[str, Any]):
        """Callback function of ColorJitter.

        Args:
            results (dict): Inputs to be transformed.

        Returns:
            dict: Dictionary that includes transformed img.
        """
        if np.random.random() < self.p:
            results["img"] = self.forward(results["img"])
        return results


@PIPELINES.register_module
class RandomGrayscale(T.RandomGrayscale):
    """Wrapper for RandomGrayscale in torchvision.transforms.

    Since this transformation is applied to PIL Image,
    `NDArrayToPILImage` must be applied first before this is applied.
    """

    def __call__(self, results: Dict[str, Any]):
        """Callback function of RandomGrayscale.

        Args:
            results (dict): Inputs to be transformed.

        Returns:
            dict: Dictionary that includes transformed img.
        """
        results["img"] = self.forward(results["img"])
        return results


@PIPELINES.register_module
class RandomGaussianBlur(T.GaussianBlur):
    """Random Gaussian Blur augmentation inherited from torchvision.transforms.GaussianBlur.

    Since this transformation is applied to PIL Image,
    `NDArrayToPILImage` must be applied first before this is applied.

    Args:
        p (float): Probability for transformation. Defaults to 0.1.
    """

    def __init__(self, p: float = 0.1, **kwargs):
        super().__init__(**kwargs)
        assert 0 <= p <= 1
        self.p = p

    def __call__(self, results: Dict[str, Any]):
        """Callback function of GaussianBlur.

        Args:
            results (dict): Inputs to be transformed.

        Returns:
            dict: Dictionary that includes transformed img.
        """
        if np.random.random() < self.p:
            results["img"] = self.forward(results["img"])
        return results


@PIPELINES.register_module
class RandomSolarization:
    """Random Solarization augmentation.

    Args:
        threshold (int): Threshold for solarization. Defaults to 128.
        p (float): Probability for transformation. Defaults to 0.2.
    """

    def __init__(self, threshold: int = 128, p: float = 0.2):
        assert 0 <= p <= 1
        self.threshold = threshold
        self.p = p

    def __call__(self, results: Dict[str, Any]):
        """Callback function of Solarization.

        Args:
            results (dict): Inputs to be transformed.

        Returns:
            dict: Dictionary that includes transformed img.
        """
        if np.random.random() < self.p:
            img = results["img"]
            img = np.where(img < self.threshold, img, 255 - img)
            results["img"] = img
        return results

    def __repr__(self):
        """Set repr of Solarization."""
        repr_str = self.__class__.__name__
        return repr_str


@PIPELINES.register_module()
class NDArrayToPILImage:
    """Convert image from numpy to PIL.

    Args:
        keys (list): Keys to be transformed.
    """

    def __init__(self, keys: List[str]):
        self.keys = keys

    def __call__(self, results: Dict[str, Any]):
        """Callback function of NDArrayToPILImage.

        Args:
            results (dict): Inputs to be transformed.

        Returns:
            dict: Dictionary that includes transformed img.
        """
        for key in self.keys:
            img = results[key]
            img = Image.fromarray(img)
            results[key] = img
        return results

    def __repr__(self):
        """Set repr of NDArrayToPILImage."""
        repr_str = self.__class__.__name__
        return repr_str


@PIPELINES.register_module()
class PILImageToNDArray:
    """Convert image from PIL to numpy.

    Args:
        keys (list): Keys to be transformed.
    """

    def __init__(self, keys: List[str]):
        self.keys = keys

    def __call__(self, results: Dict[str, Any]):
        """Callback function of PILImageToNDArray.

        Args:
            results (dict): Inputs to be transformed.

        Returns:
            dict: Dictionary that includes transformed img.
        """
        for key in self.keys:
            img = results[key]
            img = np.asarray(img)
            results[key] = img
        return results

    def __repr__(self):
        """Set repr of PILImageToNDArray."""
        repr_str = self.__class__.__name__
        return repr_str



# Copyright (C) 2021-2022 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

import pytest

from otx.api.configuration.enums.config_element_type import ConfigElementType
from otx.api.configuration.enums.utils import get_enum_names
from tests.unit.api.constants.components import OtxSdkComponent
from tests.unit.api.constants.requirements import Requirements


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestMetadataKeys:
    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_get_enum_names(self):
        """
        <b>Description:</b>
        Check "get_enum_names" function

        <b>Input data:</b>
        Enum object

        <b>Expected results:</b>
        Test passes if list returned by "get_enum_names" function is equal to expected
        """
        assert get_enum_names(ConfigElementType) == [
            "INTEGER",
            "FLOAT",
            "BOOLEAN",
            "FLOAT_SELECTABLE",
            "SELECTABLE",
            "PARAMETER_GROUP",
            "CONFIGURABLE_PARAMETERS",
            "RULE",
            "UI_RULES",
        ]



import os

import pytest

from otx.algorithms.common.configs.training_base import TrainType
from otx.core.data.adapter import get_dataset_adapter
from tests.test_suite.e2e_test_system import e2e_pytest_unit
from tests.unit.core.data.test_helpers import (
    TASK_NAME_TO_DATA_ROOT,
    TASK_NAME_TO_TASK_TYPE,
)


@e2e_pytest_unit
@pytest.mark.parametrize("task_name", TASK_NAME_TO_TASK_TYPE.keys())
@pytest.mark.parametrize("train_type", [TrainType.Incremental.value])
def test_get_dataset_adapter_incremental(task_name, train_type):
    root_path = os.getcwd()
    task_type = TASK_NAME_TO_TASK_TYPE[task_name]
    data_root = TASK_NAME_TO_DATA_ROOT[task_name]

    get_dataset_adapter(
        task_type=task_type,
        train_type=train_type,
        train_data_roots=os.path.join(root_path, data_root["train"]),
        val_data_roots=os.path.join(root_path, data_root["val"]),
    )

    get_dataset_adapter(
        task_type=task_type,
        train_type=train_type,
        test_data_roots=os.path.join(root_path, data_root["test"]),
    )


@e2e_pytest_unit
@pytest.mark.parametrize("task_name", ["classification"])
@pytest.mark.parametrize("train_type", [TrainType.Selfsupervised.value])
def test_get_dataset_adapter_selfsl_classification(task_name, train_type):
    root_path = os.getcwd()
    task_type = TASK_NAME_TO_TASK_TYPE[task_name]
    data_root = TASK_NAME_TO_DATA_ROOT[task_name]

    get_dataset_adapter(
        task_type=task_type,
        train_type=train_type,
        train_data_roots=os.path.join(root_path, data_root["train"]),
    )

    get_dataset_adapter(
        task_type=task_type,
        train_type=train_type,
        test_data_roots=os.path.join(root_path, data_root["test"]),
    )


@e2e_pytest_unit
@pytest.mark.parametrize("task_name", ["segmentation"])
@pytest.mark.parametrize("train_type", [TrainType.Selfsupervised.value])
def test_get_dataset_adapter_selfsl_segmentation(task_name, train_type):
    root_path = os.getcwd()
    task_type = TASK_NAME_TO_TASK_TYPE[task_name]
    data_root = TASK_NAME_TO_DATA_ROOT[task_name]

    get_dataset_adapter(
        task_type=task_type,
        train_type=train_type,
        train_data_roots=os.path.join(root_path, data_root["train"]),
    )

    with pytest.raises(ValueError):
        get_dataset_adapter(
            task_type=task_type,
            train_type=train_type,
            test_data_roots=os.path.join(root_path, data_root["test"]),
        )



# Copyright (C) 2023 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#
import importlib
import inspect

import pytest
from mmcv.utils import Registry

from otx.cli.utils.importing import (
    get_backbone_list,
    get_backbone_registry,
    get_impl_class,
    get_otx_root_path,
)
from tests.test_suite.e2e_test_system import e2e_pytest_unit


@e2e_pytest_unit
def test_get_impl_class():
    impl_path = "otx.algorithms.common.adapters.mmcv.models.OTXMobileNetV3"
    task_impl_class = get_impl_class(impl_path)
    assert task_impl_class.__name__ == "OTXMobileNetV3"


@e2e_pytest_unit
@pytest.mark.parametrize("backend", ["otx", "pytorchcv"])
def test_get_backbone_list(backend):
    available_backbones = get_backbone_list(backend)
    assert isinstance(available_backbones, dict)


@e2e_pytest_unit
def test_get_backbone_list_for_unsupported_backend():
    backend = "invalid"
    with pytest.raises(ValueError):
        get_backbone_list(backend)


@e2e_pytest_unit
def test_get_backbone_registry():
    backend = "otx"
    mm_registry, custom_imports = get_backbone_registry(backend)
    assert custom_imports == ["otx.algorithms.common.adapters.mmcv.models"]
    assert isinstance(mm_registry, Registry)


@e2e_pytest_unit
def test_get_backbone_registry_for_unsupported_backend():
    backend = "invalid"
    with pytest.raises(ValueError):
        get_backbone_registry(backend)


@e2e_pytest_unit
def test_get_otx_root_path(mocker):
    mocker.patch.object(importlib, "import_module", return_value=mocker.MagicMock())
    mocker.patch.object(inspect, "getfile", return_value="otx/__init__.py")
    otx_root_path = get_otx_root_path()
    assert otx_root_path == "otx"



"""NNCF wrapped mmdet models builder."""
# Copyright (C) 2022 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

from functools import partial
from typing import Optional, Union

import torch
from mmcv.parallel import DataContainer
from mmcv.runner import CheckpointLoader
from mmcv.utils import Config, ConfigDict
from mmdet.utils import get_root_logger

from otx.algorithms.common.adapters.mmcv.nncf.runners import NNCF_META_KEY
from otx.algorithms.common.adapters.mmcv.utils import (
    get_configs_by_pairs,
    remove_from_configs_by_type,
)
from otx.algorithms.common.adapters.nncf import is_accuracy_aware_training_set
from otx.algorithms.common.adapters.nncf.compression import NNCFMetaState
from otx.algorithms.common.adapters.nncf.utils import no_nncf_trace
from otx.algorithms.detection.adapters.mmdet.utils import build_detector

logger = get_root_logger()


def build_nncf_detector(  # pylint: disable=too-many-locals,too-many-statements
    config: Config,
    train_cfg: Optional[Union[Config, ConfigDict]] = None,
    test_cfg: Optional[Union[Config, ConfigDict]] = None,
    checkpoint: Optional[str] = None,
    device: Union[str, torch.device] = "cpu",
    cfg_options: Optional[Union[Config, ConfigDict]] = None,
    distributed=False,
):
    """A function to build NNCF wrapped mmdet model."""

    from mmdet.apis import multi_gpu_test, single_gpu_test
    from mmdet.apis.inference import LoadImage
    from mmdet.datasets import build_dataloader as mmdet_build_dataloader
    from mmdet.datasets import build_dataset as mmdet_build_dataset
    from mmdet.datasets.pipelines import Compose
    from nncf.torch.dynamic_graph.io_handling import nncf_model_input

    from otx.algorithms.common.adapters.mmcv.nncf.utils import (
        get_fake_input,
        model_eval,
        wrap_nncf_model,
    )
    from otx.algorithms.common.adapters.mmcv.utils.builder import (
        build_dataloader,
        build_dataset,
    )

    if checkpoint is None:
        # load model in this function not in runner
        checkpoint = config.get("load_from")
    assert checkpoint is not None, "checkpoint is not given. NNCF model must be initialized with pretrained model"

    model = build_detector(
        config,
        train_cfg=train_cfg,
        test_cfg=test_cfg,
        cfg_options=cfg_options,
        from_scratch=True,
    )
    model = model.to(device)

    state_dict = CheckpointLoader.load_checkpoint(checkpoint, map_location=device)

    is_acc_aware = is_accuracy_aware_training_set(config.get("nncf_config"))

    init_dataloader = None
    model_eval_fn = None
    if "meta" in state_dict and NNCF_META_KEY in state_dict["meta"]:
        # NNCF ckpt
        nncf_meta_state = state_dict["meta"][NNCF_META_KEY]
        data_to_build_nncf = nncf_meta_state.data_to_build
        state_to_build_nncf = nncf_meta_state.state_to_build
    else:
        # pytorch ckpt
        state_to_build_nncf = state_dict
        if "state_dict" in state_dict:
            state_to_build_nncf = state_dict["state_dict"]

        init_dataloader = build_dataloader(
            build_dataset(
                config,
                subset="train",
                dataset_builder=mmdet_build_dataset,
            ),
            config,
            subset="train",
            dataloader_builder=mmdet_build_dataloader,
            distributed=distributed,
        )

        # This data and state dict will be used to build NNCF graph later
        # when loading NNCF model
        # because some models run their subcomponents based on intermediate outputs
        # resulting differently and partially traced NNCF graph
        data_to_build_nncf = next(iter(init_dataloader))["img"]
        if isinstance(data_to_build_nncf, DataContainer):
            data_to_build_nncf = data_to_build_nncf.data[0]
        data_to_build_nncf = data_to_build_nncf.cpu().numpy()
        if len(data_to_build_nncf.shape) == 4:
            data_to_build_nncf = data_to_build_nncf[0]
        if data_to_build_nncf.shape[0] == 3:
            data_to_build_nncf = data_to_build_nncf.transpose(1, 2, 0)

        val_dataloader = None
        if is_acc_aware:
            val_dataloader = build_dataloader(
                build_dataset(
                    config,
                    subset="val",
                    dataset_builder=mmdet_build_dataset,
                ),
                config,
                subset="val",
                dataloader_builder=mmdet_build_dataloader,
                distributed=distributed,
            )

        model_eval_fn = partial(
            model_eval,
            config=config,
            val_dataloader=val_dataloader,
            evaluate_fn=multi_gpu_test if distributed else single_gpu_test,
            distributed=distributed,
        )
        state_dict = None

    test_pipeline = [LoadImage()]
    for pipeline in config.data.test.pipeline:
        if not pipeline.type.startswith("LoadImage"):
            test_pipeline.append(pipeline)

    test_pipeline = Compose(test_pipeline)
    get_fake_input_fn = partial(
        get_fake_input,
        preprocessor=test_pipeline,
        data=data_to_build_nncf,
    )

    if "nncf_compress_postprocessing" in config:
        # NB: This parameter is used to choose if we should try to make NNCF compression
        #     for a whole model graph including postprocessing (`nncf_compress_postprocessing=True`),
        #     or make NNCF compression of the part of the model without postprocessing
        #     (`nncf_compress_postprocessing=False`).
        #     Our primary goal is to make NNCF compression of such big part of the model as
        #     possible, so `nncf_compress_postprocessing=True` is our primary choice, whereas
        #     `nncf_compress_postprocessing=False` is our fallback decision.
        #     When we manage to enable NNCF compression for sufficiently many models,
        #     we should keep one choice only.
        nncf_compress_postprocessing = config.get("nncf_compress_postprocessing")
        logger.debug(f"set should_compress_postprocessing={nncf_compress_postprocessing}")
    else:
        # TODO: Do we have to keep this configuration?
        # This configuration is not enabled in forked mmdetection library in the first place
        nncf_compress_postprocessing = True

    def dummy_forward_fn(model):
        def _get_fake_data_for_forward(nncf_config):
            device = next(model.parameters()).device

            if nncf_config.get("input_info", None) and nncf_config.get("input_info").get("sample_size", None):
                input_size = nncf_config.get("input_info").get("sample_size")
                assert len(input_size) == 4 and input_size[0] == 1
                H, W, C = input_size[2], input_size[3], input_size[1]  # pylint: disable=invalid-name
                shape = tuple([H, W, C])
            else:
                shape = (128, 128, 3)

            with no_nncf_trace():
                return get_fake_input_fn(shape=shape, device=device)

        fake_data = _get_fake_data_for_forward(config.nncf_config)
        img, img_metas = fake_data["img"], fake_data["img_metas"]
        ctx = model.nncf_trace_context(img_metas, nncf_compress_postprocessing)
        with ctx:
            # The device where model is could be changed under this context
            img = [i.to(next(model.parameters()).device) for i in img]
            # Marking data as NNCF network input must be after device movement
            img = [nncf_model_input(i) for i in img]
            if nncf_compress_postprocessing:
                logger.debug("NNCF will try to compress a postprocessing part of the model")
            else:
                logger.debug("NNCF will NOT compress a postprocessing part of the model")
                img = img[0]
            model(img)

    compression_ctrl, model = wrap_nncf_model(
        config,
        model,
        model_eval_fn=model_eval_fn,
        dummy_forward_fn=dummy_forward_fn,
        dataloader_for_init=init_dataloader,
        is_accuracy_aware=is_acc_aware,
    )

    # update runner to save metadata
    config.runner.nncf_meta = NNCFMetaState(
        state_to_build=state_to_build_nncf,
        data_to_build=data_to_build_nncf,
    )

    # update custom hooks
    custom_hooks = config.get("custom_hooks", [])
    custom_hooks.append(ConfigDict({"type": "CancelTrainingHook"}))
    custom_hooks.append(
        ConfigDict(
            type="CompressionHook",
            compression_ctrl=compression_ctrl,
        )
    )
    # TODO: move this to OTX task when MPA is absorbed into OTX
    remove_from_configs_by_type(custom_hooks, "CancelInterfaceHook")
    remove_from_configs_by_type(custom_hooks, "TaskAdaptHook")
    remove_from_configs_by_type(custom_hooks, "LazyEarlyStoppingHook")
    remove_from_configs_by_type(custom_hooks, "EarlyStoppingHook")
    remove_from_configs_by_type(custom_hooks, "EMAHook")
    remove_from_configs_by_type(custom_hooks, "CustomModelEMAHook")
    config.custom_hooks = custom_hooks

    for hook in get_configs_by_pairs(custom_hooks, dict(type="OTXProgressHook")):
        time_monitor = hook.get("time_monitor", None)
        if time_monitor and getattr(time_monitor, "on_initialization_end", None) is not None:
            time_monitor.on_initialization_end()

    return compression_ctrl, model



# Copyright (C) 2021-2023 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

import pytest
import torch

from otx.algorithms.classification.adapters.mmcls.models.heads.cls_head import ClsHead
from tests.test_suite.e2e_test_system import e2e_pytest_unit


class TestClsHead:
    @pytest.fixture(autouse=True)
    def setup(self):
        self.model = ClsHead(
            do_squeeze=True,
        )

    @e2e_pytest_unit
    def test_forward_train(self):
        cls_score = torch.randn(10, 100)
        gt_label = torch.ones(10, dtype=torch.int64)
        output = self.model.forward_train(cls_score, gt_label)
        assert "loss" in output

    @e2e_pytest_unit
    def test_simple_test(self):
        cls_score = torch.randn(10, 100)
        outputs = self.model.simple_test(cls_score)
        assert isinstance(outputs, list)
        assert len(outputs) == 10



"""Utils for CLI errors."""

# Copyright (C) 2023 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#


class CliException(Exception):
    """Custom exception class for CLI."""


class ConfigValueError(CliException):
    """Configuration value is not suitable for CLI."""

    def __init__(self, message):
        super().__init__(message)


class NotSupportedError(CliException):
    """Not supported error."""

    def __init__(self, message):
        super().__init__(message)


class FileNotExistError(CliException):
    """Not exist given configuration."""

    def __init__(self, message):
        super().__init__(message)



"""OTX Adapters - mmaction.utils."""

# Copyright (C) 2022 Intel Corporation
# SPDX-License-Identifier: Apache-2.0

from .det_eval_utils import det_eval
from .export_utils import Exporter

__all__ = ["det_eval", "Exporter"]



import argparse
from pathlib import Path

import pytest

from otx.cli.tools import export as target_package
from otx.cli.tools.export import get_args, main
from tests.test_suite.e2e_test_system import e2e_pytest_unit


@e2e_pytest_unit
def test_get_args(mocker):
    mocker.patch("sys.argv", ["otx", "--load-weights", "load_weights", "--output", "output"])
    mocker.patch.object(
        target_package, "get_parser_and_hprams_data", return_value=[argparse.ArgumentParser(), "fake", "fake"]
    )

    parsed_args = get_args()

    assert parsed_args.load_weights == "load_weights"
    assert parsed_args.output == "output"


@pytest.fixture
def mock_args(mocker, tmp_dir):
    mock_args = mocker.MagicMock()
    mock_args.load_weights = "fake.bin"
    mock_args.output = tmp_dir

    def mock_contains(self, val):
        return val in self.__dict__

    mock_args.__contains__ = mock_contains
    mock_get_args = mocker.patch("otx.cli.tools.export.get_args")
    mock_get_args.return_value = mock_args

    return mock_args


@pytest.fixture
def mock_task(mocker):
    mock_task_class = mocker.MagicMock()
    mock_task = mocker.MagicMock()
    mock_task_class.return_value = mock_task
    mocker.patch.object(target_package, "get_impl_class", return_value=mock_task_class)

    return mock_task


@pytest.fixture
def mock_config_manager(mocker):
    mock_config_manager = mocker.patch.object(target_package, "ConfigManager")
    mock_template = mocker.MagicMock()
    mock_template.name = "fake_name"
    mock_config_manager.return_value.template = mock_template

    return mock_config_manager


@e2e_pytest_unit
def test_main(mocker, mock_args, mock_task, mock_config_manager, tmp_dir):
    mocker.patch.object(target_package, "is_checkpoint_nncf", return_value=True)
    mocker.patch.object(target_package, "create")
    mocker.patch.object(target_package, "TaskEnvironment")
    mocker.patch.object(target_package, "read_label_schema")
    mocker.patch.object(target_package, "read_binary")

    def mock_export_side_effect(export_type, output_model, precision, dump_features):
        output_model.set_data("fake.xml", b"fake")

    mock_task.export.side_effect = mock_export_side_effect
    tmp_dir = Path(tmp_dir)

    # run
    ret = main()

    # check
    assert ret["retcode"] == 0
    assert ret["template"] == "fake_name"
    mock_task.export.assert_called_once()
    with (tmp_dir / "fake.xml").open() as f:
        assert f.readline() == "fake"



# Copyright (C) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions
# and limitations under the License.

from operator import attrgetter

import pytest

from otx.api.entities.shapes.polygon import Point, Polygon
from otx.api.entities.shapes.rectangle import Rectangle
from otx.api.utils.time_utils import now
from tests.unit.api.constants.components import OtxSdkComponent
from tests.unit.api.constants.requirements import Requirements


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestPoint:
    def coordinates(self):
        return [0.5, 0.4]

    def other_coordinates(self):
        return [0.3, 0.2]

    def point(self):
        return Point(*self.coordinates())

    def other_point(self):
        return Point(*self.other_coordinates())

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_point_magic_methods(self):
        """
        <b>Description:</b>
        Check Point __repr__, __eq__ methods

        <b>Input data:</b>
        Initialized instance of Point

        <b>Expected results:</b>
        Test passes if Point magic methods returns correct values

        <b>Steps</b>
        1. Initialize Point instance
        2. Check returning value of magic methods
        """

        point1 = self.point()
        assert repr(point1) == "Point(0.5, 0.4)"

        point2 = self.point()
        point3 = self.other_point()
        assert point1 == point2
        assert point1 != point3
        assert point1 != str

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_point_normalize_wrt_roi(self):
        """
        <b>Description:</b>
        Check Point normalize_wrt_roi methods

        <b>Input data:</b>
        Initialized instance of Point
        Initialized instance of Rectangle

        <b>Expected results:</b>
        Test passes if Point normalize_wrt_roi returns correct values

        <b>Steps</b>
        1. Initialize Point instance
        2. Check returning value
        """

        point = self.point()
        roi = Rectangle(x1=0.3, x2=0.5, y1=0.3, y2=0.5)
        normalized = point.normalize_wrt_roi(roi)
        assert normalized.x == 0.4
        assert normalized.y == 0.38

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_point_denormalize_wrt_roi_shape(self):
        """
        <b>Description:</b>
        Check Point denormalize_wrt_roi_shape methods

        <b>Input data:</b>
        Initialized instance of Point
        Initialized instance of Rectangle

        <b>Expected results:</b>
        Test passes if Point denormalize_wrt_roi_shape returns correct values

        <b>Steps</b>
        1. Initialize Point instance
        2. Check returning value
        """

        point = self.point()
        roi = Rectangle(x1=0.4, x2=0.5, y1=0.3, y2=0.5)
        normalized = point.denormalize_wrt_roi_shape(roi)
        assert normalized.x == 1.0
        assert normalized.y == 0.5000000000000001


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestPolygon:
    modification_date = now()

    def points(self):
        point1 = Point(0.5, 0.0)
        point2 = Point(0.75, 0.2)
        point3 = Point(0.6, 0.1)
        return [point1, point2, point3]

    def other_points(self):
        point1 = Point(0.3, 0.1)
        point2 = Point(0.8, 0.3)
        point3 = Point(0.6, 0.2)
        return [point1, point2, point3]

    def polygon(self):
        return Polygon(self.points(), modification_date=self.modification_date)

    def other_polygon(self):
        return Polygon(self.other_points(), modification_date=self.modification_date)

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_polygon(self):
        """
        <b>Description:</b>
        Check Polygon parameters

        <b>Input data:</b>
        Points

        <b>Expected results:</b>
        Test passes if Polygon correctly calculates parameters and returns default values

        <b>Steps</b>
        1. Check Polygon params
        2. Check Polygon default values
        3. Check Polygon with empty points
        """

        polygon = self.polygon()
        modification_date = self.modification_date
        assert len(polygon.points) == 3
        assert polygon.modification_date == modification_date
        assert polygon.points == self.points()

        empty_points_list = []
        with pytest.raises(ValueError):
            Polygon(empty_points_list)

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_polygon_magic_methods(self):
        """
        <b>Description:</b>
        Check Polygon __repr__, __eq__, __hash__ methods

        <b>Input data:</b>
        Initialized instance of Polygon

        <b>Expected results:</b>
        Test passes if Polygon magic methods returns correct values

        <b>Steps</b>
        1. Initialize Polygon instance
        2. Check returning value of magic methods
        """

        polygon = self.polygon()
        points_len = len(self.points())
        min_x = min(self.points(), key=attrgetter("x")).x
        max_x = max(self.points(), key=attrgetter("x")).x
        min_y = min(self.points(), key=attrgetter("y")).y
        max_y = max(self.points(), key=attrgetter("y")).y

        assert f"Polygon(len(points)={points_len}, min_x={min_x}" in repr(polygon)
        assert f", max_x={max_x}, min_y={min_y}, max_y={max_y})" in repr(polygon)

        other_polygon = self.polygon()
        thirs_polygon = self.other_polygon()
        assert polygon == other_polygon
        assert polygon != thirs_polygon
        assert polygon != str

        assert hash(polygon) == hash(str(polygon))

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_polygon_normalize_wrt_roi_shape(self):
        """
        <b>Description:</b>
        Check Polygon normalize_wrt_roi_shape methods

        <b>Input data:</b>
        Initialized instance of Polygon
        Initialized instance of Rectangle

        <b>Expected results:</b>
        Test passes if Polygon normalize_wrt_roi_shape returns correct values

        <b>Steps</b>
        1. Initialize Polygon instance
        2. Check returning value
        """

        polygon = self.polygon()
        roi = Rectangle(x1=0.0, x2=0.5, y1=0.0, y2=0.5)
        normalized = polygon.normalize_wrt_roi_shape(roi)
        assert len(normalized.points) == 3
        assert normalized.min_x == 0.25
        assert normalized.max_x == 0.375
        assert normalized.min_y == 0.0
        assert normalized.max_y == 0.1

        with pytest.raises(ValueError):
            polygon.normalize_wrt_roi_shape("123")

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_polygon_denormalize_wrt_roi_shape(self):
        """
        <b>Description:</b>
        Check Polygon denormalize_wrt_roi_shape methods

        <b>Input data:</b>
        Initialized instance of Polygon
        Initialized instance of Rectangle

        <b>Expected results:</b>
        Test passes if Polygon denormalize_wrt_roi_shape returns correct values

        <b>Steps</b>
        1. Initialize Polygon instance
        2. Check returning value
        """

        polygon = self.polygon()
        roi = Rectangle(x1=0.5, x2=1.0, y1=0.0, y2=1.0)
        denormalized = polygon.denormalize_wrt_roi_shape(roi)
        assert len(denormalized.points) == 3
        assert denormalized.min_x == 0.0
        assert denormalized.max_x == 0.5
        assert denormalized.min_y == 0.0
        assert denormalized.max_y == 0.2

        with pytest.raises(ValueError):
            polygon.denormalize_wrt_roi_shape("123")

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_polygon__as_shapely_polygon(self):
        """
        <b>Description:</b>
        Check Polygon _as_shapely_polygon methods

        <b>Input data:</b>
        Initialized instance of Polygon

        <b>Expected results:</b>
        Test passes if Polygon _as_shapely_polygon returns correct values

        <b>Steps</b>
        1. Initialize Polygon instance
        2. Check returning value
        """

        polygon = self.polygon()
        polygon2 = self.other_polygon()
        shapely_polygon = polygon._as_shapely_polygon()
        shapely_polygon2 = polygon2._as_shapely_polygon()
        assert shapely_polygon.area == 0.0025000000000000022
        assert str(shapely_polygon) == "POLYGON ((0.5 0, 0.75 0.2, 0.6 0.1, 0.5 0))"
        assert shapely_polygon != shapely_polygon2

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_polygon_get_area(self):
        """
        <b>Description:</b>
        Check Polygon get_area method

        <b>Input data:</b>
        Initialized instance of Polygon

        <b>Expected results:</b>
        Test passes if Polygon get_area returns correct values

        <b>Steps</b>
        1. Initialize Polygon instance
        2. Check returning value
        """

        polygon = self.polygon()
        polygon2 = self.other_polygon()
        area = polygon.get_area()
        area2 = polygon2.get_area()
        assert area == 0.0025000000000000022
        assert area != area2



"""Initial file for mmdetection hooks."""
# Copyright (C) 2023 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

from .det_class_probability_map_hook import DetClassProbabilityMapHook

__all__ = ["DetClassProbabilityMapHook"]



import argparse

import pytest

from otx.cli.tools import eval as target_package
from otx.cli.tools.eval import get_args, main
from tests.test_suite.e2e_test_system import e2e_pytest_unit


@e2e_pytest_unit
def test_get_args(mocker):
    mock_options = {
        "--test-data-roots": "test/data/root",
        "--load-weights": "weight/path",
        "--output": "save/path",
        "--workspace": "work/dir/path",
    }
    mock_command = ["otx"]
    for key, value in mock_options.items():
        mock_command.extend([key, value])

    mocker.patch("sys.argv", mock_command)
    mocker.patch.object(
        target_package, "get_parser_and_hprams_data", return_value=[argparse.ArgumentParser(), {"param": "test"}, []]
    )
    mocker.patch.object(target_package, "add_hyper_parameters_sub_parser", return_value=argparse.ArgumentParser())

    parsed_args, _ = get_args()

    assert parsed_args.test_data_roots == "test/data/root"
    assert parsed_args.load_weights == "weight/path"
    assert parsed_args.workspace == "work/dir/path"


@pytest.fixture
def mock_args(mocker, tmp_path):
    mock_args = mocker.MagicMock()
    mock_args.test_data_roots = "fake_test_data_root"
    mock_args.load_weights = "fake_load_weights.xml"
    mock_args.workspace = tmp_path / "work_dir"

    def mock_contains(self, val):
        return val in self.__dict__

    mock_args.__contains__ = mock_contains
    mock_get_args = mocker.patch("otx.cli.tools.eval.get_args")
    mock_get_args.return_value = [mock_args, []]

    return mock_args


@pytest.fixture
def mock_config_manager(mocker):
    mock_config_manager = mocker.patch.object(target_package, "ConfigManager")
    mock_template = mocker.MagicMock()
    mock_template.name = "fake_name"
    mock_config_manager.return_value.template = mock_template
    mock_config_manager.return_value.check_workspace.return_value = True
    mock_config_manager.return_value.get_dataset_config.return_value = {}
    mock_config_manager.return_value.get_hyparams_config.return_value = {}

    return mock_config_manager


@pytest.fixture
def mock_dataset_adapter(mocker):
    mock_dataset_adapter = mocker.patch("otx.cli.tools.eval.get_dataset_adapter")
    mock_dataset = mocker.MagicMock()
    mock_label_schema = mocker.MagicMock()
    mock_dataset_adapter.return_value.get_otx_dataset.return_value = mock_dataset
    mock_dataset_adapter.return_value.get_label_schema.return_value = mock_label_schema

    return mock_dataset_adapter


@pytest.fixture
def mock_task(mocker):
    mock_task_class = mocker.MagicMock()
    mock_task = mocker.MagicMock()
    mock_task_class.return_value = mock_task
    mocker.patch.object(target_package, "get_impl_class", return_value=mock_task_class)

    return mock_task


@e2e_pytest_unit
def test_main(
    mocker,
    mock_args,
    mock_config_manager,
    mock_dataset_adapter,
):

    mocker.patch.object(
        target_package,
        "read_model",
        return_value=mocker.MagicMock(),
    )

    mocker.patch.object(
        target_package,
        "get_impl_class",
        return_value=mocker.MagicMock(),
    )

    mocker.patch.object(
        target_package,
        "get_dataset_adapter",
        return_value=mock_dataset_adapter,
    )

    mocker.patch.object(
        target_package,
        "ResultSetEntity",
        return_value=mocker.MagicMock(),
    )

    mocker.patch.object(
        target_package,
        "InferenceParameters",
        return_value=mocker.MagicMock(),
    )

    mocker.patch.object(
        target_package,
        "Subset",
        return_value=mocker.MagicMock(),
    )

    mocker.patch.object(
        target_package,
        "TaskEnvironment",
        return_value=mocker.MagicMock(),
    )
    mocker.patch("json.dump")
    mocker.patch("builtins.open")

    mock_get_args = mocker.patch("otx.cli.tools.eval.get_args")
    mock_get_args.return_value = [mock_args, []]

    ret = main()
    assert ret["retcode"] == 0



# Copyright (C) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions
# and limitations under the License.

import datetime

import pytest

from otx.api.entities.id import ID
from otx.api.entities.metrics import NullPerformance
from otx.api.entities.resultset import ResultSetEntity, ResultsetPurpose
from otx.api.utils.time_utils import now
from tests.unit.api.constants.components import OtxSdkComponent
from tests.unit.api.constants.requirements import Requirements


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestResultset:
    creation_date = now()

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_resultset_purpose(self):
        """
        <b>Description:</b>
        Check the ResultsetPurpose can correctly return the value

        <b>Input data:</b>
        Denotes, stages

        <b>Expected results:</b>
        Test passes if the results match
        """

        denotes = ["EVALUATION", "TEST", "PREEVALUATION"]
        stages = ["Validation", "Test", "Pre-validation"]

        resultset_purpose = ResultsetPurpose
        assert len(resultset_purpose) == 3

        for i in ResultsetPurpose:
            resultset_purpose = ResultsetPurpose(i)
            assert repr(resultset_purpose) == denotes[i.value]
            assert str(resultset_purpose) == stages[i.value]

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_resultset_entity(self):
        """
        <b>Description:</b>
        Check the ResultSetEntity can correctly return the value

        <b>Input data:</b>
        Mock data

        <b>Expected results:</b>
        Test passes if incoming data is processed correctly

        <b>Steps</b>
        1. Create dummy data
        2. Check the processing of default values
        3. Check the processing of changed values
        """

        test_data = {
            "model": None,
            "ground_truth_dataset": None,
            "prediction_dataset": None,
            "purpose": None,
            "performance": None,
            "creation_date": None,
            "id": None,
        }

        result_set = ResultSetEntity(**test_data)

        for name, value in test_data.items():
            set_attr_name = f"test_{name}"
            if name in [
                "model",
                "ground_truth_dataset",
                "prediction_dataset",
                "purpose",
            ]:
                assert getattr(result_set, name) == value
                setattr(result_set, name, set_attr_name)
                assert getattr(result_set, name) == set_attr_name

        assert result_set.performance == NullPerformance()
        assert type(result_set.creation_date) == datetime.datetime
        assert result_set.id_ == ID()

        assert result_set.has_score_metric() is False
        result_set.performance = "test_performance"
        assert result_set.performance != NullPerformance()
        assert result_set.has_score_metric() is True

        creation_date = self.creation_date
        result_set.creation_date = creation_date
        assert result_set.creation_date == creation_date

        set_attr_id = ID(123456789)
        result_set.id_ = set_attr_id
        assert result_set.id_ == set_attr_id

        test_result_set_repr = [
            f"model={result_set.model}",
            f"ground_truth_dataset={result_set.ground_truth_dataset}",
            f"prediction_dataset={result_set.prediction_dataset}",
            f"purpose={result_set.purpose}",
            f"performance={result_set.performance}",
            f"creation_date={result_set.creation_date}",
            f"id={result_set.id_}",
        ]

        for i in test_result_set_repr:
            assert i in repr(result_set)



"""Algorithm to find a proper batch size which is fit to current GPU device for tasks using mmcv."""

# Copyright (C) 2023 Intel Corporation
# SPDX-License-Identifier: Apache-2.0

from copy import deepcopy
from typing import Callable, Dict, List

import numpy as np

from otx.algorithms.common.adapters.torch.utils import adapt_batch_size as adapt_torch_model_bs
from otx.algorithms.common.utils.logger import get_logger

logger = get_logger()


def _set_value_at_dict_in_dict(target: Dict, key_path: str, value):
    """Set value at dictionary hierarchy structure.

    This function is for setting a value at leaf dictionary node in dictionary hierarchy structure.
    If key doesn't exist in the middle node dictionaray, then make a new dictionary at that and keep going.
    For example, if you want to set value at target["a"]["b"]["c"], then you can call the function as below.
    _set_value_at_dict_in_dict(target, "a.b.c", value)

    Args:
        target (Dict): Target variable.
        key_path (str): Dot delimited dictionary key string.
        value : Value to set.
    """
    keys = key_path.split(".")
    for key in keys[:-1]:
        if key not in target:
            target[key] = {}
        target = target[key]

    target[keys[-1]] = value


def adapt_batch_size(train_func: Callable, cfg, datasets: List, validate: bool = False):
    """Decrease batch size if default batch size isn't fit to current GPU device.

    This function just setup for single iteration training to reduce time for adapting.
    The core part of adapting batch size is done in adapt_batch_size in the torch.utils package.

    Args:
        train_func (Callable): The function to train a model.
            Only cfg, dataset and meta are passed to the function when invoking it.
        cfg: Configuration of a training.
        meta (Dict): A dict records some meta information of a training.
        datasets (List): List of datasets.
        validate (bool): Whether do vlidation or not.
    """

    def train_func_single_iter(batch_size):
        copied_cfg = deepcopy(cfg)
        _set_batch_size(copied_cfg, batch_size)

        # setup for training a single iter to reduce time
        if copied_cfg.runner.get("type") == "AccuracyAwareRunner":  # nncf case
            if "nncf_config" in copied_cfg.runner:
                _set_value_at_dict_in_dict(
                    copied_cfg.runner["nncf_config"], "accuracy_aware_training.params.maximal_total_epochs", 1
                )
        else:
            copied_cfg.runner["max_epochs"] = 1

        idx_hooks_to_remove = []
        for i, hook in enumerate(copied_cfg.custom_hooks):
            if not validate and hook["type"] == "AdaptiveTrainSchedulingHook":
                hook["enable_eval_before_run"] = False
            if hook["type"] == "OTXProgressHook" or "earlystoppinghook" in hook["type"].lower():
                idx_hooks_to_remove.append(i)

        if idx_hooks_to_remove:
            idx_hooks_to_remove.sort()
            for i in reversed(idx_hooks_to_remove):
                del copied_cfg.custom_hooks[i]

        new_datasets = [SubDataset(datasets[0], batch_size)]

        train_func(
            dataset=new_datasets,
            cfg=copied_cfg,
            validate=validate,
        )

    default_bs = _get_batch_size(cfg)

    available_bs = adapt_torch_model_bs(
        train_func=train_func_single_iter,
        current_bs=default_bs,
        trainset_size=len(datasets[0]),
    )

    if default_bs != available_bs:
        _set_batch_size(cfg, available_bs)
        origin_lr = cfg.optimizer.lr
        cfg.optimizer.lr *= available_bs / default_bs

        logger.info("Adapting batch size is done.")
        logger.info(f"Batch size is adapted : {default_bs} -> {available_bs}")
        logger.info(f"learning rate is adapted : {origin_lr} -> {cfg.optimizer.lr}")
    else:
        logger.info("Adapting batch size is done. Current batch size is availble.")


def _get_batch_size(cfg) -> int:
    if "action" in str(cfg.domain).lower():
        return cfg.data.videos_per_gpu
    return cfg.data.train_dataloader["samples_per_gpu"]


def _set_batch_size(cfg, batch_size: int):
    if "action" in str(cfg.domain).lower():
        cfg.data.videos_per_gpu = batch_size
    else:
        cfg.data.train_dataloader["samples_per_gpu"] = batch_size


class SubDataset:
    """Wrapper class to make dataset pretend to have specified number of images.

    Args:
        fullset: Original dataset.
        num_samples (int): Number of images to pretend to have. It should be positive.
    """

    def __init__(self, fullset, num_sampels: int):
        if num_sampels <= 0:
            raise ValueError(f"num_sampels should be positive. But, current value is {num_sampels}.")

        self.fullset = fullset
        self.num_sampels = num_sampels

    def __len__(self) -> int:
        """Get length of subset."""
        return self.num_sampels

    def __getitem__(self, indx) -> dict:
        """Get dataset at index."""
        return self.fullset[indx]

    def __getattr__(self, name):
        """When trying to get other attributes, not dataset, get values from fullset."""
        if name == "__setstate__":
            raise AttributeError(name)
        return getattr(self.fullset, name)

    @property
    def flag(self):
        """Getter of flag for detection task.

        Sampler of the detection task decides length of dataset checking sum of flag array.
        To consider that case, return flag array with length of num_samples.

        """
        return np.zeros(self.num_sampels, dtype=np.uint8)



"""Task of OTX Segmentation."""

# Copyright (C) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions
# and limitations under the License.

import io
import os
from abc import ABC, abstractmethod
from typing import List, Optional

import numpy as np
import torch
from mmcv.utils import ConfigDict

from otx.algorithms.common.configs.training_base import TrainType
from otx.algorithms.common.tasks.base_task import TRAIN_TYPE_DIR_PATH, OTXTask
from otx.algorithms.common.utils.callback import (
    InferenceProgressCallback,
    TrainingProgressCallback,
)
from otx.algorithms.common.utils.logger import get_logger
from otx.algorithms.segmentation.adapters.openvino.model_wrappers.blur import (
    get_activation_map,
)
from otx.algorithms.segmentation.configs.base import SegmentationConfig
from otx.api.configuration import cfg_helper
from otx.api.configuration.helper.utils import ids_to_strings
from otx.api.entities.datasets import DatasetEntity
from otx.api.entities.explain_parameters import ExplainParameters
from otx.api.entities.inference_parameters import InferenceParameters
from otx.api.entities.inference_parameters import (
    default_progress_callback as default_infer_progress_callback,
)
from otx.api.entities.metrics import (
    CurveMetric,
    InfoMetric,
    LineChartInfo,
    MetricsGroup,
    Performance,
    ScoreMetric,
    VisualizationInfo,
    VisualizationType,
)
from otx.api.entities.model import (
    ModelEntity,
    ModelFormat,
    ModelOptimizationType,
    ModelPrecision,
)
from otx.api.entities.result_media import ResultMediaEntity
from otx.api.entities.resultset import ResultSetEntity
from otx.api.entities.task_environment import TaskEnvironment
from otx.api.entities.tensor import TensorEntity
from otx.api.entities.train_parameters import TrainParameters, default_progress_callback
from otx.api.serialization.label_mapper import label_schema_to_bytes
from otx.api.usecases.evaluation.metrics_helper import MetricsHelper
from otx.api.usecases.tasks.interfaces.export_interface import ExportType
from otx.api.utils.segmentation_utils import (
    create_annotation_from_segmentation_map,
    create_hard_prediction_from_soft_prediction,
)
from otx.cli.utils.multi_gpu import is_multigpu_child_process

logger = get_logger()
RECIPE_TRAIN_TYPE = {
    TrainType.Semisupervised: "semisl.py",
    TrainType.Incremental: "incremental.py",
    TrainType.Selfsupervised: "selfsl.py",
}


class OTXSegmentationTask(OTXTask, ABC):
    """Task class for OTX segmentation."""

    # pylint: disable=too-many-instance-attributes, too-many-locals
    def __init__(self, task_environment: TaskEnvironment, output_path: Optional[str] = None):
        super().__init__(task_environment, output_path)
        self._task_config = SegmentationConfig
        self._hyperparams: ConfigDict = task_environment.get_hyper_parameters(self._task_config)
        self._model_name = task_environment.model_template.name
        self._train_type = self._hyperparams.algo_backend.train_type
        self.metric = "mDice"
        self._label_dictionary = dict(enumerate(sorted(self._labels), 1))

        self._model_dir = os.path.join(
            os.path.abspath(os.path.dirname(self._task_environment.model_template.model_template_path)),
            TRAIN_TYPE_DIR_PATH[self._train_type.name],
        )
        if (
            self._train_type in RECIPE_TRAIN_TYPE
            and self._train_type == TrainType.Incremental
            and self._hyperparams.learning_parameters.enable_supcon
            and not self._model_dir.endswith("supcon")
        ):
            self._model_dir = os.path.join(self._model_dir, "supcon")

        if task_environment.model is not None:
            self._load_model()

        self.data_pipeline_path = os.path.join(self._model_dir, "data_pipeline.py")

    def _load_model_ckpt(self, model: Optional[ModelEntity]):
        if model and "weights.pth" in model.model_adapters:
            # If a model has been trained and saved for the task already, create empty model and load weights here
            buffer = io.BytesIO(model.get_data("weights.pth"))
            model_data = torch.load(buffer, map_location=torch.device("cpu"))
            return model_data
        return None

    def infer(
        self,
        dataset: DatasetEntity,
        inference_parameters: Optional[InferenceParameters] = None,
    ) -> DatasetEntity:
        """Main infer function."""
        logger.info("infer()")

        if inference_parameters is not None:
            update_progress_callback = inference_parameters.update_progress
            is_evaluation = inference_parameters.is_evaluation
        else:
            update_progress_callback = default_infer_progress_callback
            is_evaluation = False

        update_progress_callback = default_progress_callback
        if inference_parameters is not None:
            update_progress_callback = inference_parameters.update_progress  # type: ignore

        self._time_monitor = InferenceProgressCallback(len(dataset), update_progress_callback)

        predictions = self._infer_model(dataset, InferenceParameters(is_evaluation=True))
        prediction_results = zip(predictions["eval_predictions"], predictions["feature_vectors"])
        self._add_predictions_to_dataset(prediction_results, dataset, dump_soft_prediction=not is_evaluation)

        logger.info("Inference completed")
        return dataset

    def train(
        self, dataset: DatasetEntity, output_model: ModelEntity, train_parameters: Optional[TrainParameters] = None
    ):
        """Train function for OTX segmentation task.

        Actual training is processed by _train_model fucntion
        """
        logger.info("train()")
        # Check for stop signal when training has stopped.
        # If should_stop is true, training was cancelled and no new
        if self._should_stop:  # type: ignore
            logger.info("Training cancelled.")
            self._should_stop = False
            self._is_training = False
            return

        # Set OTX LoggerHook & Time Monitor
        if train_parameters:
            update_progress_callback = train_parameters.update_progress
        else:
            update_progress_callback = default_progress_callback
        self._time_monitor = TrainingProgressCallback(update_progress_callback)

        results = self._train_model(dataset)

        # Check for stop signal when training has stopped. If should_stop is true, training was cancelled and no new
        if self._should_stop:
            logger.info("Training cancelled.")
            self._should_stop = False
            self._is_training = False
            return

        # get output model
        model_ckpt = results.get("final_ckpt")
        if model_ckpt is None:
            logger.error("cannot find final checkpoint from the results.")
            # output_model.model_status = ModelStatus.FAILED
            return
        # update checkpoint to the newly trained model
        self._model_ckpt = model_ckpt

        # get prediction on validation set
        self._is_training = False

        # Get training metrics group from learning curves
        training_metrics, best_score = self._generate_training_metrics(self._learning_curves)
        performance = Performance(
            score=ScoreMetric(value=best_score, name=self.metric),
            dashboard_metrics=training_metrics,
        )

        logger.info(f"Final model performance: {str(performance)}")
        # save resulting model
        self.save_model(output_model)
        output_model.performance = performance
        self._is_training = False
        logger.info("train done.")

    def export(
        self,
        export_type: ExportType,
        output_model: ModelEntity,
        precision: ModelPrecision = ModelPrecision.FP32,
        dump_features: bool = True,
    ):
        """Export function of OTX Task."""
        logger.info("Exporting the model")
        if export_type != ExportType.OPENVINO:
            raise RuntimeError(f"not supported export type {export_type}")
        output_model.model_format = ModelFormat.OPENVINO
        output_model.optimization_type = ModelOptimizationType.MO

        results = self._export_model(precision, dump_features)
        outputs = results.get("outputs")
        logger.debug(f"results of run_task = {outputs}")
        if outputs is None:
            raise RuntimeError(results.get("msg"))

        bin_file = outputs.get("bin")
        xml_file = outputs.get("xml")
        onnx_file = outputs.get("onnx")

        if xml_file is None or bin_file is None or onnx_file is None:
            raise RuntimeError("invalid status of exporting. bin and xml or onnx should not be None")
        with open(bin_file, "rb") as f:
            output_model.set_data("openvino.bin", f.read())
        with open(xml_file, "rb") as f:
            output_model.set_data("openvino.xml", f.read())
        with open(onnx_file, "rb") as f:
            output_model.set_data("model.onnx", f.read())
        output_model.precision = self._precision
        output_model.optimization_methods = self._optimization_methods
        output_model.has_xai = dump_features
        output_model.set_data("label_schema.json", label_schema_to_bytes(self._task_environment.label_schema))
        logger.info("Exporting completed")

    def explain(
        self,
        dataset: DatasetEntity,
        explain_parameters: Optional[ExplainParameters] = None,
    ) -> DatasetEntity:
        """Main explain function of OTX Task."""
        raise NotImplementedError

    def evaluate(
        self,
        output_resultset: ResultSetEntity,
        evaluation_metric: Optional[str] = None,
    ):
        """Evaluate function of OTX Segmentation Task."""
        logger.info("called evaluate()")
        if evaluation_metric is not None:
            logger.warning(
                f"Requested to use {evaluation_metric} metric, " "but parameter is ignored. Use mDice instead."
            )
        metric = MetricsHelper.compute_dice_averaged_over_pixels(output_resultset)
        logger.info(f"mDice after evaluation: {metric.overall_dice.value}")
        output_resultset.performance = metric.get_performance()
        logger.info("Evaluation completed")

    def _add_predictions_to_dataset(self, prediction_results, dataset, dump_soft_prediction):
        """Loop over dataset again to assign predictions. Convert from MMSegmentation format to OTX format."""
        for dataset_item, (prediction, feature_vector) in zip(dataset, prediction_results):
            soft_prediction = np.transpose(prediction[0], axes=(1, 2, 0))
            hard_prediction = create_hard_prediction_from_soft_prediction(
                soft_prediction=soft_prediction,
                soft_threshold=self._hyperparams.postprocessing.soft_threshold,
                blur_strength=self._hyperparams.postprocessing.blur_strength,
            )
            annotations = create_annotation_from_segmentation_map(
                hard_prediction=hard_prediction,
                soft_prediction=soft_prediction,
                label_map=self._label_dictionary,
            )
            dataset_item.append_annotations(annotations=annotations)

            if feature_vector is not None:
                active_score = TensorEntity(name="representation_vector", numpy=feature_vector.reshape(-1))
                dataset_item.append_metadata_item(active_score, model=self._task_environment.model)

            if dump_soft_prediction:
                for label_index, label in self._label_dictionary.items():
                    if label_index == 0:
                        continue
                    current_label_soft_prediction = soft_prediction[:, :, label_index]
                    class_act_map = get_activation_map(current_label_soft_prediction)
                    result_media = ResultMediaEntity(
                        name=label.name,
                        type="soft_prediction",
                        label=label,
                        annotation_scene=dataset_item.annotation_scene,
                        roi=dataset_item.roi,
                        numpy=class_act_map,
                    )
                    dataset_item.append_metadata_item(result_media, model=self._task_environment.model)

    def save_model(self, output_model: ModelEntity):
        """Save best model weights in SegmentationTrainTask."""

        if is_multigpu_child_process():
            return
        logger.info("called save_model")
        buffer = io.BytesIO()
        hyperparams_str = ids_to_strings(cfg_helper.convert(self._hyperparams, dict, enum_to_str=True))
        labels = {label.name: label.color.rgb_tuple for label in self._labels}
        model_ckpt = torch.load(self._model_ckpt)
        modelinfo = {
            "model": model_ckpt,
            "config": hyperparams_str,
            "labels": labels,
            "VERSION": 1,
        }

        torch.save(modelinfo, buffer)
        output_model.set_data("weights.pth", buffer.getvalue())
        output_model.set_data(
            "label_schema.json",
            label_schema_to_bytes(self._task_environment.label_schema),
        )
        output_model.precision = self._precision

    def _generate_training_metrics(self, learning_curves):
        """Get Training metrics (epochs & scores).

        Parses the mmsegmentation logs to get metrics from the latest training run
        :return output List[MetricsGroup]
        """
        output: List[MetricsGroup] = []
        # Model architecture
        architecture = InfoMetric(name="Model architecture", value=self._model_name)
        visualization_info_architecture = VisualizationInfo(
            name="Model architecture", visualisation_type=VisualizationType.TEXT
        )
        output.append(
            MetricsGroup(
                metrics=[architecture],
                visualization_info=visualization_info_architecture,
            )
        )
        # Learning curves
        best_score = -1
        for key, curve in learning_curves.items():
            metric_curve = CurveMetric(xs=curve.x, ys=curve.y, name=key)
            if key == f"val/{self.metric}":
                best_score = max(curve.y)
            visualization_info = LineChartInfo(name=key, x_axis_label="Epoch", y_axis_label=key)
            output.append(MetricsGroup(metrics=[metric_curve], visualization_info=visualization_info))

        return output, best_score

    @abstractmethod
    def _train_model(self, dataset: DatasetEntity):
        """Train model and return the results."""
        raise NotImplementedError

    @abstractmethod
    def _infer_model(
        self,
        dataset: DatasetEntity,
        inference_parameters: Optional[InferenceParameters] = None,
    ):
        """Get inference results from dataset."""
        raise NotImplementedError

    @abstractmethod
    def _export_model(self, precision, dump_features):
        """Export model and return the results."""
        raise NotImplementedError

    @abstractmethod
    def _explain_model(self, dataset: DatasetEntity, explain_parameters: Optional[ExplainParameters]):
        """Explain model and return the results."""
        raise NotImplementedError
